@@ -1,5 +1,5 @@
 /* Copyright (c) 2009, 2013, Oracle and/or its affiliates.
-   Copyright (c) 2013, 2022, MariaDB
+   Copyright (c) 2009, 2020, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -121,6 +121,25 @@ static void init_debug_sync_psi_keys(void)
 #endif /* HAVE_PSI_INTERFACE */
 
 
+/**
+  Set the THD::proc_info without instrumentation.
+  This method is private to DEBUG_SYNC,
+  and on purpose avoid any use of:
+  - the SHOW PROFILE instrumentation
+  - the PERFORMANCE_SCHEMA instrumentation
+  so that using DEBUG_SYNC() in the server code
+  does not cause the instrumentations to record
+  spurious data.
+*/
+static const char*
+debug_sync_thd_proc_info(THD *thd, const char* info)
+{
+  const char* old_proc_info= thd->proc_info;
+  thd->proc_info= info;
+  return old_proc_info;
+}
+
+
 /**
   Initialize the debug sync facility at server start.
 
@@ -236,7 +255,8 @@ void debug_sync_init_thread(THD *thd)
   if (opt_debug_sync_timeout)
   {
     thd->debug_sync_control= (st_debug_sync_control*)
-      my_malloc(sizeof(st_debug_sync_control),
+      my_malloc(PSI_NOT_INSTRUMENTED,
+                sizeof(st_debug_sync_control),
                 MYF(MY_WME | MY_ZEROFILL | MY_THREAD_SPECIFIC));
     if (!thd->debug_sync_control)
     {
@@ -267,12 +287,6 @@ void debug_sync_end_thread(THD *thd)
   {
     st_debug_sync_control *ds_control= thd->debug_sync_control;
 
-    /*
-      This synchronization point can be used to synchronize on thread end.
-      This is the latest point in a THD's life, where this can be done.
-    */
-    DEBUG_SYNC(thd, "thread_end");
-
     if (ds_control->ds_action)
     {
       st_debug_sync_action *action= ds_control->ds_action;
@@ -302,6 +316,20 @@ void debug_sync_end_thread(THD *thd)
 }
 
 
+void debug_sync_reset_thread(THD *thd)
+{
+  if (thd->debug_sync_control)
+  {
+    /*
+      This synchronization point can be used to synchronize on thread end.
+      This is the latest point in a THD's life, where this can be done.
+    */
+    DEBUG_SYNC(thd, "thread_end");
+    thd->debug_sync_control->ds_active= 0;
+  }
+}
+
+
 /**
   Move a string by length.
 
@@ -320,13 +348,12 @@ static char *debug_sync_bmove_len(char *to, char *to_end,
   DBUG_ASSERT(to_end);
   DBUG_ASSERT(!length || from);
   set_if_smaller(length, (size_t) (to_end - to));
-  if (length)
-    memcpy(to, from, length);
+  memcpy(to, from, length);
   return (to + length);
 }
 
 
-#ifdef DBUG_TRACE
+#if !defined(DBUG_OFF)
 
 /**
   Create a string that describes an action.
@@ -416,7 +443,8 @@ static void debug_sync_print_actions(THD *thd)
 
   DBUG_VOID_RETURN;
 }
-#endif /* defined(DBUG_TRACE) */
+
+#endif /* !defined(DBUG_OFF) */
 
 
 /**
@@ -657,7 +685,7 @@ static st_debug_sync_action *debug_sync_get_action(THD *thd,
     if (ds_control->ds_active > ds_control->ds_allocated)
     {
       uint new_alloc= ds_control->ds_active + 3;
-      void *new_action= my_realloc(ds_control->ds_action,
+      void *new_action= my_realloc(PSI_NOT_INSTRUMENTED, ds_control->ds_action,
                                    new_alloc * sizeof(st_debug_sync_action),
                                    MYF(MY_WME | MY_ALLOW_ZERO_PTR));
       if (!new_action)
@@ -747,7 +775,6 @@ static bool debug_sync_set_action(THD *thd, st_debug_sync_action *action)
   else
   {
     const char *dsp_name= action->sync_point.c_ptr();
-#ifdef DBUG_TRACE
     DBUG_EXECUTE("debug_sync", {
         /* Functions as DBUG_PRINT args can change keyword and line nr. */
         const char *sig_emit= action->signal.c_ptr();
@@ -758,7 +785,6 @@ static bool debug_sync_set_action(THD *thd, st_debug_sync_action *action)
                     dsp_name, action->activation_count,
                     action->hit_limit, action->execute, action->timeout,
                     sig_emit, sig_wait));});
-#endif
 
     /* Check this before sorting the array. action may move. */
     is_dsp_now= !my_strcasecmp(system_charset_info, dsp_name, "now");
@@ -771,9 +797,7 @@ static bool debug_sync_set_action(THD *thd, st_debug_sync_action *action)
                sizeof(st_debug_sync_action), debug_sync_qsort_cmp);
     }
   }
-#ifdef DBUG_TRACE
   DBUG_EXECUTE("debug_sync_list", debug_sync_print_actions(thd););
-#endif
 
   /* Execute the special sync point 'now' if activated above. */
   if (is_dsp_now)
@@ -860,8 +884,7 @@ static char *debug_sync_token(char **token_p, uint *token_length_p,
   DBUG_ASSERT(ptr);
 
   /* Skip leading space */
-  ptr+= system_charset_info->cset->scan(system_charset_info,
-                                        ptr, ptrend, MY_SEQ_SPACES);
+  ptr+= system_charset_info->scan(ptr, ptrend, MY_SEQ_SPACES);
   if (!*ptr)
   {
     ptr= NULL;
@@ -872,8 +895,7 @@ static char *debug_sync_token(char **token_p, uint *token_length_p,
   *token_p= ptr;
 
   /* Find token end. */
-  ptr+= system_charset_info->cset->scan(system_charset_info,
-                                        ptr, ptrend, MY_SEQ_NONSPACES);
+  ptr+= system_charset_info->scan(ptr, ptrend, MY_SEQ_NONSPACES);
 
   /* Get token length. */
   *token_length_p= (uint)(ptr - *token_p);
@@ -883,7 +905,7 @@ static char *debug_sync_token(char **token_p, uint *token_length_p,
   {
     DBUG_ASSERT(ptr < ptrend);
     /* Get terminator character length. */
-    uint mbspacelen= my_charlen_fix(system_charset_info, ptr, ptrend);
+    uint mbspacelen= system_charset_info->charlen_fix(ptr, ptrend);
 
     /* Terminate token. */
     *ptr= '\0';
@@ -892,8 +914,7 @@ static char *debug_sync_token(char **token_p, uint *token_length_p,
     ptr+= mbspacelen;
 
     /* Skip trailing space */
-    ptr+= system_charset_info->cset->scan(system_charset_info,
-                                          ptr, ptrend, MY_SEQ_SPACES);
+    ptr+= system_charset_info->scan(ptr, ptrend, MY_SEQ_SPACES);
   }
 
  end:
@@ -1337,7 +1358,7 @@ uchar *debug_sync_value_ptr(THD *thd)
 
 static void debug_sync_execute(THD *thd, st_debug_sync_action *action)
 {
-#ifdef DBUG_TRACE
+#ifndef DBUG_OFF
   const char *dsp_name= action->sync_point.c_ptr();
   const char *sig_emit= action->signal.c_ptr();
   const char *sig_wait= action->wait_for.c_ptr();
@@ -1371,7 +1392,7 @@ static void debug_sync_execute(THD *thd, st_debug_sync_action *action)
       strxnmov(ds_control->ds_proc_info, sizeof(ds_control->ds_proc_info)-1,
                "debug sync point: ", action->sync_point.c_ptr(), NullS);
       old_proc_info= thd->proc_info;
-      thd_proc_info(thd, ds_control->ds_proc_info);
+      debug_sync_thd_proc_info(thd, ds_control->ds_proc_info);
     }
 
     /*
@@ -1428,12 +1449,12 @@ static void debug_sync_execute(THD *thd, st_debug_sync_action *action)
         restore_current_mutex = false;
 
       set_timespec(abstime, action->timeout);
-      DBUG_EXECUTE("debug_sync_exec",
+      DBUG_EXECUTE("debug_sync_exec", {
           /* Functions as DBUG_PRINT args can change keyword and line nr. */
+          const char *sig_glob= debug_sync_global.ds_signal.c_ptr();
           DBUG_PRINT("debug_sync_exec",
                      ("wait for '%s'  at: '%s'  curr: '%s'",
-                      sig_wait, dsp_name,
-                      debug_sync_global.ds_signal.c_ptr())););
+                      sig_wait, dsp_name, sig_glob));});
 
       /*
         Wait until global signal string matches the wait_for string.
@@ -1447,21 +1468,19 @@ static void debug_sync_execute(THD *thd, st_debug_sync_action *action)
         error= mysql_cond_timedwait(&debug_sync_global.ds_cond,
                                     &debug_sync_global.ds_mutex,
                                     &abstime);
-        DBUG_EXECUTE("debug_sync",
+        DBUG_EXECUTE("debug_sync", {
             /* Functions as DBUG_PRINT args can change keyword and line nr. */
+            const char *sig_glob= debug_sync_global.ds_signal.c_ptr();
             DBUG_PRINT("debug_sync",
                        ("awoke from %s  global: %s  error: %d",
-                        sig_wait, debug_sync_global.ds_signal.c_ptr(),
-                        error)););
+                        sig_wait, sig_glob, error));});
         if (unlikely(error == ETIMEDOUT || error == ETIME))
         {
           // We should not make the statement fail, even if in strict mode.
-          const bool save_abort_on_warning= thd->abort_on_warning;
-          thd->abort_on_warning= false;
+          Abort_on_warning_instant_set aws(thd, false);
           push_warning(thd, Sql_condition::WARN_LEVEL_WARN,
                        ER_DEBUG_SYNC_TIMEOUT,
                        ER_THD(thd, ER_DEBUG_SYNC_TIMEOUT));
-          thd->abort_on_warning= save_abort_on_warning;
           DBUG_EXECUTE_IF("debug_sync_abort_on_timeout", DBUG_ASSERT(0););
           break;
         }
@@ -1491,11 +1510,11 @@ static void debug_sync_execute(THD *thd, st_debug_sync_action *action)
         mysql_mutex_lock(&thd->mysys_var->mutex);
         thd->mysys_var->current_mutex= old_mutex;
         thd->mysys_var->current_cond= old_cond;
-        thd_proc_info(thd, old_proc_info);
+        debug_sync_thd_proc_info(thd, old_proc_info);
         mysql_mutex_unlock(&thd->mysys_var->mutex);
       }
       else
-        thd_proc_info(thd, old_proc_info);
+        debug_sync_thd_proc_info(thd, old_proc_info);
     }
     else
     {
