@@ -23,6 +23,36 @@
 #include "sql_parse.h"
 #include "sql_cte.h"
 
+
+/**
+  @brief
+    Walk through all VALUES items.
+  @param
+     @param processor      - the processor to call for each Item
+     @param walk_qubquery  - if should dive into subquery items
+     @param argument       - the argument to pass recursively
+  @retval
+    true   on error
+    false  on success
+*/
+bool table_value_constr::walk_values(Item_processor processor,
+                                     bool walk_subquery,
+                                     void *argument)
+{
+  List_iterator_fast<List_item> list_item_it(lists_of_values);
+  while (List_item *list= list_item_it++)
+  {
+    List_iterator_fast<Item> item_it(*list);
+    while (Item *item= item_it++)
+    {
+       if (item->walk(&Item::unknown_splocal_processor, false, argument))
+         return true;
+    }
+  }
+  return false;
+}
+
+
 /**
   @brief
     Fix fields for TVC values
@@ -47,7 +77,7 @@ bool fix_fields_for_tvc(THD *thd, List_iterator_fast<List_item> &li)
 
   while ((lst= li++))
   {
-    List_iterator<Item> it(*lst);
+    List_iterator_fast<Item> it(*lst);
     Item *item;
 
     while ((item= it++))
@@ -59,7 +89,7 @@ bool fix_fields_for_tvc(THD *thd, List_iterator_fast<List_item> &li)
         while replacing their values to NAME_CONST()s.
         So fix only those that have not been.
       */
-      if (item->fix_fields_if_needed_for_scalar(thd, it.ref()) ||
+      if (item->fix_fields_if_needed(thd, 0) ||
           item->check_is_evaluable_expression_or_error())
 	DBUG_RETURN(true);
     }
@@ -341,13 +371,6 @@ int table_value_constr::save_explain_data_intern(THD *thd,
   if (select_lex->master_unit()->derived)
     explain->connection_type= Explain_node::EXPLAIN_NODE_DERIVED;
 
-  for (SELECT_LEX_UNIT *unit= select_lex->first_inner_unit();
-       unit;
-       unit= unit->next_unit())
-  {
-    explain->add_child(unit->first_select()->select_number);
-  }
-
   output->add_node(explain);
 
   if (select_lex->is_top_level_node())
@@ -372,14 +395,9 @@ bool table_value_constr::optimize(THD *thd)
       thd->lex->explain && // for "SET" command in SPs.
       (!thd->lex->explain->get_select(select_lex->select_number)))
   {
-    if (save_explain_data_intern(thd, thd->lex->explain))
-      return true;
+    return save_explain_data_intern(thd, thd->lex->explain);
   }
-
-  if (select_lex->optimize_unflattened_subqueries(true))
-    return true;
-
-  return false;
+  return 0;
 }
 
 
@@ -406,11 +424,10 @@ bool table_value_constr::exec(SELECT_LEX *sl)
 
   while ((elem= li++))
   {
-    THD *cur_thd= sl->parent_lex->thd;
-    if (send_records >= sl->master_unit()->select_limit_cnt)
+    if (send_records >= sl->master_unit()->lim.get_select_limit())
       break;
-    int rc= result->send_data(*elem);
-    cur_thd->get_stmt_da()->inc_current_row_for_warning();
+    int rc=
+      result->send_data_with_check(*elem, sl->master_unit(), send_records);
     if (!rc)
       send_records++;
     else if (rc > 0)
@@ -540,10 +557,7 @@ bool Item_func_in::create_value_list_for_tvc(THD *thd,
 
     if (is_list_of_rows)
     {
-      Item_row *row_list= (Item_row *)(args[i]->build_clone(thd));
-
-      if (!row_list)
-        return true;
+      Item_row *row_list= (Item_row *)(args[i]);
 
       for (uint j=0; j < row_list->cols(); j++)
       {
@@ -565,8 +579,7 @@ bool Item_func_in::create_value_list_for_tvc(THD *thd,
         sprintf(col_name, "_col_%i", 1);
         args[i]->set_name(thd, col_name, strlen(col_name), thd->charset());
       }
-      Item *arg_clone= args[i]->build_clone(thd);
-      if (!arg_clone || tvc_value->push_back(arg_clone))
+      if (tvc_value->push_back(args[i]->real_item()))
         return true;
     }
 
@@ -653,69 +666,50 @@ st_select_lex *wrap_tvc(THD *thd, st_select_lex *tvc_sl,
                         st_select_lex *parent_select)
 {
   LEX *lex= thd->lex;
-  select_result *save_result= lex->result;
+  select_result *save_result= thd->lex->result;
   uint8 save_derived_tables= lex->derived_tables;
   thd->lex->result= NULL;
 
   Query_arena backup;
   Query_arena *arena= thd->activate_stmt_arena_if_needed(&backup);
-
+  /*
+    Create SELECT_LEX of the select used in the result of transformation
+  */
+  lex->current_select= tvc_sl;
+  if (mysql_new_select(lex, 0, NULL))
+    goto err;
+  mysql_init_select(lex);
+  /* Create item list as '*' for the subquery SQ */
   Item *item;
   SELECT_LEX *wrapper_sl;
-  SELECT_LEX_UNIT *derived_unit;
-
-  /*
-    Create SELECT_LEX wrapper_sl of the select used in the result
-    of the transformation
-  */
-  if (!(wrapper_sl= new (thd->mem_root) SELECT_LEX()))
-    goto err;
-  wrapper_sl->select_number= ++thd->lex->stmt_lex->current_select_number;
-  wrapper_sl->parent_lex= lex; /* Used in init_query. */
-  wrapper_sl->init_query();
-  wrapper_sl->init_select();
-
-  wrapper_sl->nest_level= tvc_sl->nest_level;
-  wrapper_sl->parsing_place= tvc_sl->parsing_place;
+  wrapper_sl= lex->current_select;
   wrapper_sl->set_linkage(tvc_sl->get_linkage());
-  wrapper_sl->exclude_from_table_unique_test=
-                                 tvc_sl->exclude_from_table_unique_test;
-
-  lex->current_select= wrapper_sl;
+  wrapper_sl->parsing_place= SELECT_LIST;
   item= new (thd->mem_root) Item_field(thd, &wrapper_sl->context,
-                                       NULL, NULL, &star_clex_str);
+                                       star_clex_str);
   if (item == NULL || add_item_to_list(thd, item))
     goto err;
   (wrapper_sl->with_wild)++;
-
-  /* Include the newly created select into the global list of selects */
-  wrapper_sl->include_global((st_select_lex_node**)&lex->all_selects_list);
-
-  /* Substitute select node used of TVC for the newly created select */
-  tvc_sl->substitute_in_tree(wrapper_sl);
-
+  
+  /* Exclude SELECT with TVC */
+  tvc_sl->exclude();
   /*
-    Create a unit for the substituted select used for TVC and attach it
-    to the the wrapper select wrapper_sl as the only unit. The created
-    unit is the unit for the derived table tvc_x of the transformation.
+    Create derived table DT that will wrap TVC in the result of transformation
   */
-  if (!(derived_unit= new (thd->mem_root) SELECT_LEX_UNIT()))
+  SELECT_LEX *tvc_select; // select for tvc
+  SELECT_LEX_UNIT *derived_unit; // unit for tvc_select
+  if (mysql_new_select(lex, 1, tvc_sl))
     goto err;
-  derived_unit->init_query();
-  derived_unit->thd= thd;
-  derived_unit->include_down(wrapper_sl);
+  tvc_select= lex->current_select;
+  derived_unit= tvc_select->master_unit();
+  tvc_select->set_linkage(DERIVED_TABLE_TYPE);
+
+  lex->current_select= wrapper_sl;
 
   /*
-    Attach the select used of TVC as the only slave to the unit for
-    the derived table tvc_x of the transformation
-  */
-  derived_unit->add_slave(tvc_sl);
-  tvc_sl->set_linkage(DERIVED_TABLE_TYPE);
-
-  /*
-    Generate the name of the derived table created for TVC and
-    add it to the FROM list of the wrapping select
-  */
+    Create the name of the wrapping derived table and
+    add it to the FROM list of the wrapper
+   */
   Table_ident *ti;
   LEX_CSTRING alias;
   TABLE_LIST *derived_tab;
@@ -734,15 +728,19 @@ st_select_lex *wrap_tvc(THD *thd, st_select_lex *tvc_sl,
   wrapper_sl->table_list.first->derived_type= DTYPE_TABLE | DTYPE_MATERIALIZE;
   lex->derived_tables|= DERIVED_SUBQUERY;
 
+  wrapper_sl->where= 0;
+  wrapper_sl->set_braces(false);
+  derived_unit->set_with_clause(0);
+
   if (arena)
     thd->restore_active_arena(arena, &backup);
-  lex->result= save_result;
+  thd->lex->result= save_result;
   return wrapper_sl;
 
 err:
   if (arena)
     thd->restore_active_arena(arena, &backup);
-  lex->result= save_result;
+  thd->lex->result= save_result;
   lex->derived_tables= save_derived_tables;
   return 0;
 }
@@ -792,6 +790,7 @@ st_select_lex *wrap_tvc_with_tail(THD *thd, st_select_lex *tvc_sl)
   {
     wrapper_sl->master_unit()->union_distinct= wrapper_sl;
   }
+  wrapper_sl->distinct= tvc_sl->distinct;
   thd->lex->current_select= wrapper_sl;
   return wrapper_sl;
 }
@@ -811,12 +810,11 @@ st_select_lex *wrap_tvc_with_tail(THD *thd, st_select_lex *tvc_sl)
     SELECT * FROM (VALUES (v1), ... (vn)) tvc_x
     and replaces the subselect with the result of the transformation.
 
-  @retval wrapping select if successful
-          0  otherwise
+  @retval false if successfull
+          true  otherwise
 */
 
-st_select_lex *
-Item_subselect::wrap_tvc_into_select(THD *thd, st_select_lex *tvc_sl)
+bool Item_subselect::wrap_tvc_into_select(THD *thd, st_select_lex *tvc_sl)
 {
   LEX *lex= thd->lex;
   /* SELECT_LEX object where the transformation is performed */
@@ -826,9 +824,14 @@ Item_subselect::wrap_tvc_into_select(THD *thd, st_select_lex *tvc_sl)
   {
     if (engine->engine_type() == subselect_engine::SINGLE_SELECT_ENGINE)
       ((subselect_single_select_engine *) engine)->change_select(wrapper_sl);
+    lex->current_select= wrapper_sl;
+    return false;
+  }
+  else
+  {
+    lex->current_select= parent_select;
+    return true;
   }
-  lex->current_select= parent_select;
-  return wrapper_sl;
 }
 
 
@@ -857,8 +860,7 @@ static bool cmp_row_types(Item* item1, Item* item2)
     Item *inner= item1->element_index(i);
     Item *outer= item2->element_index(i);
     if (!inner->type_handler()->subquery_type_allows_materialization(inner,
-                                                                     outer,
-                                                                     true))
+                                                                     outer))
       return true;
   }
   return false;
@@ -901,6 +903,8 @@ Item *Item_func_in::in_predicate_to_in_subs_transformer(THD *thd,
   if (!transform_into_subq)
     return this;
   
+  transform_into_subq= false;
+
   List<List_item> values;
 
   LEX *lex= thd->lex;
@@ -922,7 +926,7 @@ Item *Item_func_in::in_predicate_to_in_subs_transformer(THD *thd,
   
   for (uint i=1; i < arg_count; i++)
   {
-    if (!args[i]->const_item() || cmp_row_types(args[i], args[0]))
+    if (!args[i]->const_item() || cmp_row_types(args[0], args[i]))
       return this;
   }
 
@@ -941,7 +945,7 @@ Item *Item_func_in::in_predicate_to_in_subs_transformer(THD *thd,
   sq_select= lex->current_select;
   sq_select->parsing_place= SELECT_LIST;
   item= new (thd->mem_root) Item_field(thd, &sq_select->context,
-                                       NULL, NULL, &star_clex_str);
+                                       star_clex_str);
   if (item == NULL || add_item_to_list(thd, item))
     goto err;
   (sq_select->with_wild)++;
@@ -1002,7 +1006,6 @@ Item *Item_func_in::in_predicate_to_in_subs_transformer(THD *thd,
   if (!(in_subs=
           new (thd->mem_root) Item_in_subselect(thd, args[0], sq_select)))
     goto err;
-  in_subs->converted_from_in_predicate= TRUE;
   sq= in_subs;
   if (negated)
     sq= negate_expression(thd, in_subs);
@@ -1057,38 +1060,15 @@ uint32 Item_func_in::max_length_of_left_expr()
 
 bool Item_func_in::to_be_transformed_into_in_subq(THD *thd)
 {
-  bool is_row_list= args[1]->type() == Item::ROW_ITEM;
   uint values_count= arg_count-1;
 
-  if (is_row_list)
+  if (args[1]->type() == Item::ROW_ITEM)
     values_count*= ((Item_row *)(args[1]))->cols();
 
   if (thd->variables.in_subquery_conversion_threshold == 0 ||
       thd->variables.in_subquery_conversion_threshold > values_count)
     return false;
 
-  if (!(thd->lex->context_analysis_only & CONTEXT_ANALYSIS_ONLY_PREPARE))
-    return true;
-
-  /* Occurence of '?' in IN list is checked only for PREPARE <stmt> commands */
-  for (uint i=1; i < arg_count; i++)
-  {
-    if (!is_row_list)
-    {
-      if (args[i]->type() == Item::PARAM_ITEM)
-        return false;
-    }
-    else
-    {
-      Item_row *row_list= (Item_row *)(args[i]);
-      for (uint j=0; j < row_list->cols(); j++)
-      {
-        if (row_list->element_index(j)->type() == Item::PARAM_ITEM)
-          return false;
-      }
-    }
-  }
-
   return true;
 }
 
