@@ -30,14 +30,17 @@ ulong   wsrep_reject_queries;
 
 int wsrep_init_vars()
 {
-  wsrep_provider        = my_strdup(WSREP_NONE, MYF(MY_WME));
-  wsrep_provider_options= my_strdup("", MYF(MY_WME));
-  wsrep_cluster_address = my_strdup("", MYF(MY_WME));
-  wsrep_cluster_name    = my_strdup(WSREP_CLUSTER_NAME, MYF(MY_WME));
-  wsrep_node_name       = my_strdup("", MYF(MY_WME));
-  wsrep_node_address    = my_strdup("", MYF(MY_WME));
-  wsrep_node_incoming_address= my_strdup(WSREP_NODE_INCOMING_AUTO, MYF(MY_WME));
-  wsrep_start_position  = my_strdup(WSREP_START_POSITION_ZERO, MYF(MY_WME));
+  wsrep_provider        = my_strdup(PSI_INSTRUMENT_ME, WSREP_NONE, MYF(MY_WME));
+  wsrep_provider_options= my_strdup(PSI_INSTRUMENT_ME, "", MYF(MY_WME));
+  wsrep_cluster_address = my_strdup(PSI_INSTRUMENT_ME, "", MYF(MY_WME));
+  wsrep_cluster_name    = my_strdup(PSI_INSTRUMENT_ME, WSREP_CLUSTER_NAME, MYF(MY_WME));
+  wsrep_node_name       = my_strdup(PSI_INSTRUMENT_ME, "", MYF(MY_WME));
+  wsrep_node_address    = my_strdup(PSI_INSTRUMENT_ME, "", MYF(MY_WME));
+  wsrep_node_incoming_address= my_strdup(PSI_INSTRUMENT_ME, WSREP_NODE_INCOMING_AUTO, MYF(MY_WME));
+  if (wsrep_gtid_mode)
+    wsrep_start_position  = my_strdup(PSI_INSTRUMENT_ME, WSREP_START_POSITION_ZERO_GTID, MYF(MY_WME));
+  else
+    wsrep_start_position  = my_strdup(PSI_INSTRUMENT_ME, WSREP_START_POSITION_ZERO, MYF(MY_WME));
   return 0;
 }
 
@@ -47,7 +50,7 @@ static int get_provider_option_value(const char* opts,
 {
   int ret= 1;
   ulong opt_value_tmp;
-  char *opt_value_str, *s, *opts_copy= my_strdup(opts, MYF(MY_WME));
+  char *opt_value_str, *s, *opts_copy= my_strdup(PSI_INSTRUMENT_ME, opts, MYF(MY_WME));
 
   if ((opt_value_str= strstr(opts_copy, opt_name)) == NULL)
     goto end;
@@ -179,6 +182,13 @@ bool wsrep_sync_wait_update (sys_var* self, THD* thd, enum_var_type var_type)
   return false;
 }
 
+template<typename T>
+static T parse_value(char** startptr, char** endptr)
+{
+   T val= strtoll(*startptr, *&endptr, 10);
+   *startptr= *endptr;
+   return val;
+}
 
 /*
   Verify the format of the given UUID:seqno.
@@ -212,8 +222,25 @@ bool wsrep_start_position_verify (const char* start_str)
     return true;
 
   char* endptr;
+  char* startptr= (char *)start_str + uuid_len + 1;
   wsrep_seqno_t const seqno __attribute__((unused)) // to avoid GCC warnings
-    (strtoll(&start_str[uuid_len + 1], &endptr, 10));
+    (parse_value<uint64_t>(&startptr, &endptr));
+
+  // Start parsing native GTID part
+  if (*startptr == ',')
+  {
+    startptr++;
+    uint32_t domain  __attribute__((unused))
+      (parse_value<uint32_t>(&startptr, &endptr));
+    if (*endptr != '-') return true;
+    startptr++;
+    uint32_t server  __attribute__((unused))
+      (parse_value<uint32_t>(&startptr, &endptr));
+    if (*endptr != '-') return true;
+    startptr++;
+    uint64_t seq  __attribute__((unused))
+      (parse_value<uint64_t>(&startptr, &endptr));
+  }
 
   // Remaining string was seqno.
   if (*endptr == '\0') return false;
@@ -226,9 +253,22 @@ static
 bool wsrep_set_local_position(THD* thd, const char* const value,
                               size_t length, bool const sst)
 {
+  char* endptr;
+  char* startptr;
   wsrep_uuid_t uuid;
   size_t const uuid_len= wsrep_uuid_scan(value, length, &uuid);
-  wsrep_seqno_t const seqno= strtoll(value + uuid_len + 1, NULL, 10);
+  startptr= (char *)value + uuid_len + 1;
+  wsrep_seqno_t const seqno= parse_value<uint64_t>(&startptr, &endptr);
+
+  if (*startptr == ',')
+  {
+    startptr++;
+    wsrep_gtid_server.domain_id= parse_value<uint32_t>(&startptr, &endptr);
+    startptr++;
+    wsrep_gtid_server.server_id= parse_value<uint32_t>(&startptr, &endptr);
+    startptr++;
+    wsrep_gtid_server.seqno(parse_value<uint64_t>(&startptr, &endptr));
+  }
 
   if (sst) {
     wsrep_sst_received (thd, uuid, seqno, NULL, 0);
@@ -320,12 +360,6 @@ static int wsrep_provider_verify (const char* provider_str)
   {
     return 1;
   }
-
-  if (MY_S_ISDIR(f_stat.st_mode))
-  {
-    return 1;
-  }
-
   return 0;
 }
 
@@ -411,7 +445,7 @@ void wsrep_provider_init (const char* value)
   }
 
   if (wsrep_provider) my_free((void *)wsrep_provider);
-  wsrep_provider= my_strdup(value, MYF(0));
+  wsrep_provider= my_strdup(PSI_INSTRUMENT_MEM, value, MYF(0));
   wsrep_set_wsrep_on();
 }
 
@@ -442,7 +476,7 @@ void wsrep_provider_options_init(const char* value)
 {
   if (wsrep_provider_options && wsrep_provider_options != value) 
     my_free((void *)wsrep_provider_options);
-  wsrep_provider_options= (value) ? my_strdup(value, MYF(0)) : NULL;
+  wsrep_provider_options= value ? my_strdup(PSI_INSTRUMENT_MEM, value, MYF(0)) : NULL;
 }
 
 bool wsrep_reject_queries_update(sys_var *self, THD* thd, enum_var_type type)
@@ -473,6 +507,15 @@ bool wsrep_debug_update(sys_var *self, THD* thd, enum_var_type type)
     return false;
 }
 
+bool
+wsrep_gtid_seq_no_check(sys_var *self, THD *thd, set_var *var)
+{
+  ulonglong new_wsrep_gtid_seq_no= var->save_result.ulonglong_value;
+  if (wsrep_gtid_mode && new_wsrep_gtid_seq_no > wsrep_gtid_server.seqno())
+    return false;
+  return true;
+}
+
 static int wsrep_cluster_address_verify (const char* cluster_address_str)
 {
   /* There is no predefined address format, it depends on provider. */
@@ -545,8 +588,8 @@ void wsrep_cluster_address_init (const char* value)
               (wsrep_cluster_address) ? wsrep_cluster_address : "null", 
               (value) ? value : "null");
 
-  my_free((void*) wsrep_cluster_address);
-  wsrep_cluster_address= my_strdup(value ? value : "", MYF(0));
+  my_free(const_cast<char*>(wsrep_cluster_address));
+  wsrep_cluster_address= my_strdup(PSI_INSTRUMENT_MEM, safe_str(value), MYF(0));
 }
 
 /* wsrep_cluster_name cannot be NULL or an empty string. */
@@ -619,7 +662,7 @@ void wsrep_node_address_init (const char* value)
   if (wsrep_node_address && strcmp(wsrep_node_address, value))
     my_free ((void*)wsrep_node_address);
 
-  wsrep_node_address= (value) ? my_strdup(value, MYF(0)) : NULL;
+  wsrep_node_address= value ? my_strdup(PSI_INSTRUMENT_MEM, value, MYF(0)) : NULL;
 }
 
 static void wsrep_slave_count_change_update ()
@@ -889,8 +932,7 @@ static void export_wsrep_status_to_mysql(THD* thd)
   mysql_status_vars[wsrep_status_len].type  = SHOW_LONG;
 }
 
-int wsrep_show_status (THD *thd, SHOW_VAR *var, void *,
-                       system_status_var *, enum_var_type)
+int wsrep_show_status (THD *thd, SHOW_VAR *var, char *buff)
 {
   /* Note that we should allow show status like 'wsrep%' even
   when WSREP(thd) is false. */
