@@ -11,7 +11,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
+	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
 /** @file ha_connect.h
 	Author Olivier Bertrand
@@ -104,9 +104,7 @@ struct ha_field_option_struct
   uint opt;
   const char *dateformat;
   const char *fieldformat;
-	const char* jsonpath;
-	const char* xmlpath;
-	char *special;
+  char *special;
 };
 
 /*
@@ -146,7 +144,7 @@ typedef class ha_connect *PHC;
 /** @brief
   Class definition for the storage engine
 */
-class ha_connect: public handler
+class ha_connect final : public handler
 {
   THR_LOCK_DATA lock;      ///< MySQL lock
   CONNECT_SHARE *share;        ///< Shared lock info
@@ -390,7 +388,7 @@ virtual int check(THD* thd, HA_CHECK_OPT* check_opt);
     We implement this in ha_connect.cc. It's not an obligatory method;
     skip it and and MySQL will treat it as not implemented.
   */
-  int write_row(uchar *buf);
+  int write_row(const uchar *buf);
 
   /** @brief
     We implement this in ha_connect.cc. It's not an obligatory method;
@@ -463,8 +461,8 @@ int index_prev(uchar *buf);
   int start_stmt(THD *thd, thr_lock_type lock_type);
   int external_lock(THD *thd, int lock_type);                   ///< required
   int delete_all_rows(void);
-  ha_rows records_in_range(uint inx, key_range *min_key,
-                           key_range *max_key);
+  ha_rows records_in_range(uint inx, const key_range *start_key,
+                           const key_range *end_key, page_range *pages);
   /**
     These methods can be overridden, but their default implementation
     provide useful functionality.
@@ -551,3 +549,7 @@ public:
   uint    int_table_flags;            // Inherited from MyISAM
   bool    enable_activate_all_index;  // Inherited from MyISAM
 };  // end of ha_connect class definition
+
+#if defined(JAVA_SUPPORT) || defined(CMGO_SUPPORT)
+bool MongoEnabled(void);
+#endif   // JAVA_SUPPORT || CMGO_SUPPORT
