@@ -1,22 +1,28 @@
 /************** tabrest C++ Program Source Code File (.CPP) ************/
-/* PROGRAM NAME: tabrest   Version 2.0                                 */
-/*  (C) Copyright to the author Olivier BERTRAND          2018 - 2021  */
+/* PROGRAM NAME: tabrest   Version 1.7                                 */
+/*  (C) Copyright to the author Olivier BERTRAND          2018 - 2019  */
 /*  This program is the REST Web API support for MariaDB.              */
-/*  The way Connect handles NOSQL data returned by REST queries is     */
-/*  just by retrieving it as a file and then leave the existing data   */
-/*  type tables (JSON, XML or CSV) process it as usual.                */
+/*  When compiled without MARIADB defined, it is the EOM module code.  */
 /***********************************************************************/
 
 /***********************************************************************/
 /*  Definitions needed by the included files.                          */
 /***********************************************************************/
+#if defined(MARIADB)
 #include <my_global.h>    // All MariaDB stuff
-#include <mysqld.h>
-#include <sql_error.h>
-#if !defined(__WIN__) && !defined(_WINDOWS)
-#include <sys/types.h>
-#include <sys/wait.h>
-#endif	 // !__WIN__ && !_WINDOWS
+#else   // !MARIADB       OEM module
+#include "mini-global.h"
+#define _MAX_PATH 260
+#if !defined(REST_SOURCE)
+#if defined(__WIN__) || defined(_WINDOWS)
+#include <windows.h>
+#else		 // !__WIN__
+#define __stdcall
+#include <dlfcn.h>         // dlopen(), dlclose(), dlsym() ...
+#endif   // !__WIN__
+#endif	 // !REST_SOURCE
+#define _OS_H_INCLUDED     // Prevent os.h to be called
+#endif  // !MARIADB
 
 /***********************************************************************/
 /*  Include application header files:                                  */
@@ -36,114 +42,44 @@
 #include "tabfmt.h"
 #include "tabrest.h"
 
-#if defined(connect_EXPORTS)
-#define PUSH_WARNING(M) push_warning(current_thd, Sql_condition::WARN_LEVEL_WARN, 0, M)
-#else
-#define PUSH_WARNING(M) htrc(M)
-#endif
-
 static XGETREST getRestFnc = NULL;
-static int Xcurl(PGLOBAL g, PCSZ Http, PCSZ Uri, PCSZ filename);
+
+#if !defined(MARIADB)
+/***********************************************************************/
+/*  DB static variables.                                               */
+/***********************************************************************/
+int    TDB::Tnum;
+int    DTVAL::Shift;
+int    CSORT::Limit = 0;
+double CSORT::Lg2 = log(2.0);
+size_t CSORT::Cpn[1000] = { 0 };
 
 /***********************************************************************/
-/*  Xcurl: retrieve the REST answer by executing cURL.                 */
+/*  These functions are exported from the REST library.                */
 /***********************************************************************/
-int Xcurl(PGLOBAL g, PCSZ Http, PCSZ Uri, PCSZ filename)
+extern "C" {
+  PTABDEF __stdcall GetREST(PGLOBAL, void*);
+  PQRYRES __stdcall ColREST(PGLOBAL, PTOS, char*, char*, bool);
+} // extern "C"
+
+/***********************************************************************/
+/*  This function returns a table definition class.                    */
+/***********************************************************************/
+PTABDEF __stdcall GetREST(PGLOBAL g, void *memp)
 {
-	char  buf[512];
-	int   rc = 0;
-
-	if (strchr(filename, '"')) {
-		strcpy(g->Message, "Invalid file name");
-		return 1;
-	} // endif filename
-
-	if (Uri) {
-		if (*Uri == '/' || Http[strlen(Http) - 1] == '/')
-			sprintf(buf, "%s%s", Http, Uri);
-		else
-			sprintf(buf, "%s/%s", Http, Uri);
-
-	} else
-		strcpy(buf, Http);
-
-#if defined(__WIN__)
-	char cmd[1024];
-	STARTUPINFO si;
-	PROCESS_INFORMATION pi;
-
-	sprintf(cmd, "curl \"%s\" -o \"%s\"", buf, filename);
-
-	ZeroMemory(&si, sizeof(si));
-	si.cb = sizeof(si);
-	ZeroMemory(&pi, sizeof(pi));
-
-	// Start the child process. 
-	if (CreateProcess(NULL, cmd, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi)) {
-		// Wait until child process exits.
-		WaitForSingleObject(pi.hProcess, INFINITE);
-
-		// Close process and thread handles. 
-		CloseHandle(pi.hProcess);
-		CloseHandle(pi.hThread);
-	} else {
-		sprintf(g->Message, "CreateProcess curl failed (%d)", GetLastError());
-		rc = 1;
-	}	// endif CreateProcess
-#else   // !__WIN__
-	char  fn[600];
-	pid_t pID;
-
-	// Check if curl package is availabe by executing subprocess
-	FILE *f= popen("command -v curl", "r");
-
-	if (!f) {
-			strcpy(g->Message, "Problem in allocating memory.");
-			return 1;
-	} else {
-		char   temp_buff[50];
-		size_t len = fread(temp_buff,1, 50, f);
-
-		if(!len) {
-			strcpy(g->Message, "Curl not installed.");
-			return 1;
-		}	else
-			pclose(f);
-
-	} // endif f
-	
-	pID = vfork();
-	sprintf(fn, "-o%s", filename);
-
-	if (pID == 0) {
-		// Code executed by child process
-		execlp("curl", "curl", buf, fn, (char*)NULL);
-
-		// If execlp() is successful, we should not reach this next line.
-		strcpy(g->Message, "Unsuccessful execlp from vfork()");
-		exit(1);
-	} else if (pID < 0) {
-		// failed to fork
-		strcpy(g->Message, "Failed to fork");
-		rc = 1;
-	} else {
-		// Parent process
-		wait(NULL);  // Wait for the child to terminate
-	}	// endif pID
-#endif  // !__WIN__
-
-	return rc;
-} // end of Xcurl
+  return new(g, memp) RESTDEF;
+} // end of GetREST
+#endif   // !MARIADB
 
 /***********************************************************************/
-/*  GetREST: load the Rest lib and get the Rest function.              */
+/*  GetREST: get the external TABDEF from OEM module.                  */
 /***********************************************************************/
 XGETREST GetRestFunction(PGLOBAL g)
 {
 	if (getRestFnc)
 		return getRestFnc;
 	
-#if !defined(REST_SOURCE)
+#if !defined(MARIADB) || !defined(REST_SOURCE)
 	if (trace(515))
 		htrc("Looking for GetRest library\n");
 
@@ -196,9 +132,9 @@ XGETREST GetRestFunction(PGLOBAL g)
 		return NULL;
 	} // endif getdef
 #endif  // !__WIN__
-#else   // REST_SOURCE
+#else
 	getRestFnc = restGetFile;
-#endif	// REST_SOURCE
+#endif
 
 	return getRestFnc;
 } // end of GetRestFunction
@@ -206,53 +142,37 @@ XGETREST GetRestFunction(PGLOBAL g)
 /***********************************************************************/
 /*  Return the columns definition to MariaDB.                          */
 /***********************************************************************/
+#if defined(MARIADB)
 PQRYRES RESTColumns(PGLOBAL g, PTOS tp, char *tab, char *db, bool info)
+#else   // !MARIADB
+PQRYRES __stdcall ColREST(PGLOBAL g, PTOS tp, char *tab, char *db, bool info)
+#endif  // !MARIADB
 {
-  PQRYRES  qrp= NULL;
-  char     filename[_MAX_PATH + 1];  // MAX PATH ???
-	int      rc;
-  PCSZ     http, uri, fn, ftype;
-	XGETREST grf = NULL;
-	bool     curl = GetBooleanTableOption(g, tp, "Curl", false);
+  PQRYRES qrp= NULL;
+  char filename[_MAX_PATH + 1];  // MAX PATH ???
+  PCSZ http, uri, fn, ftype;
+	XGETREST grf = GetRestFunction(g);
 
-	if (!curl && !(grf = GetRestFunction(g)))
-		curl = true;
+	if (!grf)
+		return NULL;
 
   http = GetStringTableOption(g, tp, "Http", NULL);
   uri = GetStringTableOption(g, tp, "Uri", NULL);
+  fn = GetStringTableOption(g, tp, "Filename", "rest.json");
+#if defined(MARIADB)
   ftype = GetStringTableOption(g, tp, "Type", "JSON");
-	fn = GetStringTableOption(g, tp, "Filename", NULL);
-
-	if (!fn) {
-		int n, m = strlen(ftype) + 1;
-
-		strcat(strcpy(filename, tab), ".");
-		n = strlen(filename);
-
-		// Fold ftype to lower case
-		for (int i = 0; i < m; i++)
-			filename[n + i] = tolower(ftype[i]);
-
-		fn = filename;
-		tp->subtype = PlugDup(g, fn);
-		sprintf(g->Message, "No file name. Table will use %s", fn);
-		PUSH_WARNING(g->Message);
-	}	// endif fn
+#else   // !MARIADB
+  // OEM tables must specify the file type
+  ftype = GetStringTableOption(g, tp, "Ftype", "JSON");
+#endif  // !MARIADB
 
   //  We used the file name relative to recorded datapath
-	PlugSetPath(filename, fn, db);
-	remove(filename);
+  snprintf(filename, sizeof filename, IF_WIN(".\\%s\\%s","./%s/%s"), db, fn);
 
   // Retrieve the file from the web and copy it locally
-	if (curl)
-		rc = Xcurl(g, http, uri, filename);
-	else
-		rc = grf(g->Message, trace(515), http, uri, filename);
-
-	if (rc) {
-		strcpy(g->Message, "Cannot access to curl nor casablanca");
-		return NULL;
-	} else if (!stricmp(ftype, "JSON"))
+	if (http && grf(g->Message, trace(515), http, uri, filename)) {
+			// sprintf(g->Message, "Failed to get file at %s", http);
+  } else if (!stricmp(ftype, "JSON"))
     qrp = JSONColumns(g, db, NULL, tp, info);
   else if (!stricmp(ftype, "CSV"))
     qrp = CSVColumns(g, NULL, tp, info);
@@ -273,17 +193,21 @@ PQRYRES RESTColumns(PGLOBAL g, PTOS tp, char *tab, char *db, bool info)
 /***********************************************************************/
 bool RESTDEF::DefineAM(PGLOBAL g, LPCSTR am, int poff)
 {
-	char     filename[_MAX_PATH + 1];
-  int      rc = 0, n;
-	bool     xt = trace(515);
-	LPCSTR   ftype;
-	XGETREST grf = NULL;
-	bool     curl = GetBoolCatInfo("Curl", false);
+	char    filename[_MAX_PATH + 1];
+  int     rc = 0, n;
+	bool    xt = trace(515);
+	LPCSTR  ftype;
+	XGETREST grf = GetRestFunction(g);
 
-	if (!curl && !(grf = GetRestFunction(g)))
-		curl = true;
+	if (!grf)
+		return true;
 
+#if defined(MARIADB)
   ftype = GetStringCatInfo(g, "Type", "JSON");
+#else   // !MARIADB
+  // OEM tables must specify the file type
+  ftype = GetStringCatInfo(g, "Ftype", "JSON");
+#endif  // !MARIADB
 
   if (xt)
     htrc("ftype = %s am = %s\n", ftype, SVP(am));
@@ -295,32 +219,29 @@ bool RESTDEF::DefineAM(PGLOBAL g, LPCSTR am, int poff)
     : (!stricmp(ftype, "CSV"))  ? 3 : 0;
 
   if (n == 0) {
-    htrc("DefineAM: Unsupported REST table type %s\n", ftype);
-    sprintf(g->Message, "Unsupported REST table type %s", ftype);
+    htrc("DefineAM: Unsupported REST table type %s", am);
+    sprintf(g->Message, "Unsupported REST table type %s", am);
     return true;
   } // endif n
 
   Http = GetStringCatInfo(g, "Http", NULL);
   Uri = GetStringCatInfo(g, "Uri", NULL);
-  Fn = GetStringCatInfo(g, "Filename", NULL);
+  Fn = GetStringCatInfo(g, "Filename", "rest.json");
 
   //  We used the file name relative to recorded datapath
-  PlugSetPath(filename, Fn, GetPath());
-	remove(filename);
+  //PlugSetPath(filename, Fn, GetPath());
+  strcpy(filename, GetPath());
+	strncat(filename, Fn, _MAX_PATH - strlen(filename));
 
   // Retrieve the file from the web and copy it locally
-	if (curl) {
-		rc = Xcurl(g, Http, Uri, filename);
-		xtrc(515, "Return from Xcurl: rc=%d\n", rc);
-	} else {
-		rc = grf(g->Message, xt, Http, Uri, filename);
-		xtrc(515, "Return from restGetFile: rc=%d\n", rc);
-	} // endelse
+	rc = grf(g->Message, xt, Http, Uri, filename);
 
-	if (rc) {
-		// strcpy(g->Message, "Cannot access to curl nor casablanca");
-		return true;
-	} else switch (n) {
+  if (xt)
+    htrc("Return from restGetFile: rc=%d\n", rc);
+
+  if (rc)
+    return true;
+  else switch (n) {
     case 1: Tdp = new (g) JSONDEF; break;
 #if defined(XML_SUPPORT)
 		case 2: Tdp = new (g) XMLDEF;  break;
@@ -348,7 +269,7 @@ PTDB RESTDEF::GetTable(PGLOBAL g, MODE m)
   if (trace(515))
     htrc("REST GetTable mode=%d\n", m);
 
-  if (m != MODE_READ && m != MODE_READX && m != MODE_ANY) {
+  if (m != MODE_READ && m != MODE_READX) {
     strcpy(g->Message, "REST tables are currently read only");
     return NULL;
   } // endif m
