@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2019, 2022, MariaDB Corporation.
+Copyright (c) 2018, 2020, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -87,9 +87,8 @@ rtr_page_split_initialize_nodes(
 	stop = task + n_recs;
 
 	rec = page_rec_get_next(page_get_infimum_rec(page));
-	const ulint n_core = page_is_leaf(page)
-		? cursor->index->n_core_fields : 0;
-	*offsets = rec_get_offsets(rec, cursor->index, *offsets, n_core,
+	const bool is_leaf = page_is_leaf(page);
+	*offsets = rec_get_offsets(rec, cursor->index, *offsets, is_leaf,
 				   n_uniq, &heap);
 
 	source_cur = rec_get_nth_field(rec, *offsets, 0, &len);
@@ -102,7 +101,7 @@ rtr_page_split_initialize_nodes(
 
 		rec = page_rec_get_next(rec);
 		*offsets = rec_get_offsets(rec, cursor->index, *offsets,
-					   n_core, n_uniq, &heap);
+					   is_leaf, n_uniq, &heap);
 		source_cur = rec_get_nth_field(rec, *offsets, 0, &len);
 	}
 
@@ -184,82 +183,6 @@ rtr_index_build_node_ptr(
 	return(tuple);
 }
 
-/**************************************************************//**
-In-place update the mbr field of a spatial index row.
-@return true if update is successful */
-static
-bool
-rtr_update_mbr_field_in_place(
-/*==========================*/
-	dict_index_t*	index,		/*!< in: spatial index. */
-	rec_t*		rec,		/*!< in/out: rec to be modified.*/
-	rec_offs*	offsets,	/*!< in/out: offsets on rec. */
-	rtr_mbr_t*	mbr,		/*!< in: the new mbr. */
-	mtr_t*		mtr)		/*!< in: mtr */
-{
-	void*		new_mbr_ptr;
-	double		new_mbr[SPDIMS * 2];
-	byte*		log_ptr;
-	page_t*		page = page_align(rec);
-	ulint		len = DATA_MBR_LEN;
-	ulint		flags = BTR_NO_UNDO_LOG_FLAG
-			| BTR_NO_LOCKING_FLAG
-			| BTR_KEEP_SYS_FLAG;
-	ulint		rec_info;
-
-	rtr_write_mbr(reinterpret_cast<byte*>(&new_mbr), mbr);
-	new_mbr_ptr = static_cast<void*>(new_mbr);
-	/* Otherwise, set the mbr to the new_mbr. */
-	rec_set_nth_field(rec, offsets, 0, new_mbr_ptr, len);
-
-	rec_info = rec_get_info_bits(rec, rec_offs_comp(offsets));
-
-	/* Write redo log. */
-	/* For now, we use LOG_REC_UPDATE_IN_PLACE to log this enlarge.
-	In the future, we may need to add a new log type for this. */
-	log_ptr = mlog_open_and_write_index(mtr, rec, index, page_is_comp(page)
-					    ? MLOG_COMP_REC_UPDATE_IN_PLACE
-					    : MLOG_REC_UPDATE_IN_PLACE,
-					    1 + DATA_ROLL_PTR_LEN + 14 + 2
-					    + MLOG_BUF_MARGIN);
-
-	if (!log_ptr) {
-		/* Logging in mtr is switched off during
-		crash recovery */
-		return(false);
-	}
-
-	/* Flags */
-	mach_write_to_1(log_ptr, flags);
-	log_ptr++;
-	/* TRX_ID Position */
-	log_ptr += mach_write_compressed(log_ptr, 0);
-	/* ROLL_PTR */
-	trx_write_roll_ptr(log_ptr, 0);
-	log_ptr += DATA_ROLL_PTR_LEN;
-	/* TRX_ID */
-	log_ptr += mach_u64_write_compressed(log_ptr, 0);
-
-	/* Offset */
-	mach_write_to_2(log_ptr, page_offset(rec));
-	log_ptr += 2;
-	/* Info bits */
-	mach_write_to_1(log_ptr, rec_info);
-	log_ptr++;
-	/* N fields */
-	log_ptr += mach_write_compressed(log_ptr, 1);
-	/* Field no, len */
-	log_ptr += mach_write_compressed(log_ptr, 0);
-	log_ptr += mach_write_compressed(log_ptr, len);
-	/* Data */
-	memcpy(log_ptr, new_mbr_ptr, len);
-	log_ptr += len;
-
-	mlog_close(mtr, log_ptr);
-
-	return(true);
-}
-
 /**************************************************************//**
 Update the mbr field of a spatial index row.
 @return true if update is successful */
@@ -281,7 +204,7 @@ rtr_update_mbr_field(
 	mem_heap_t*	heap;
 	page_t*		page;
 	rec_t*		rec;
-	ulint		flags = BTR_NO_UNDO_LOG_FLAG
+	constexpr ulint flags = BTR_NO_UNDO_LOG_FLAG
 			| BTR_NO_LOCKING_FLAG
 			| BTR_KEEP_SYS_FLAG;
 	dberr_t		err;
@@ -292,7 +215,6 @@ rtr_update_mbr_field(
 	ulint		low_match = 0;
 	ulint		child;
 	ulint		rec_info;
-	page_zip_des_t*	page_zip;
 	bool		ins_suc = true;
 	ulint		cur2_pos = 0;
 	ulint		del_page_no = 0;
@@ -306,11 +228,9 @@ rtr_update_mbr_field(
 	heap = mem_heap_create(100);
 	block = btr_cur_get_block(cursor);
 	ut_ad(page == buf_block_get_frame(block));
-	page_zip = buf_block_get_page_zip(block);
 
 	child = btr_node_ptr_get_child_page_no(rec, offsets);
-	const ulint n_core = page_is_leaf(block->frame)
-		? index->n_core_fields : 0;
+	const bool is_leaf = page_is_leaf(block->frame);
 
 	if (new_rec) {
 		child_rec = new_rec;
@@ -326,17 +246,22 @@ rtr_update_mbr_field(
 	if (cursor2) {
 		rec_t*	del_rec = btr_cur_get_rec(cursor2);
 		offsets2 = rec_get_offsets(btr_cur_get_rec(cursor2),
-					   index, NULL, 0,
+					   index, NULL, false,
 					   ULINT_UNDEFINED, &heap);
 		del_page_no = btr_node_ptr_get_child_page_no(del_rec, offsets2);
 		cur2_pos = page_rec_get_n_recs_before(btr_cur_get_rec(cursor2));
 	}
 
+	ut_ad(rec_offs_validate(rec, index, offsets));
+	ut_ad(rec_offs_base(offsets)[0 + 1] == DATA_MBR_LEN);
+	ut_ad(node_ptr->fields[0].len == DATA_MBR_LEN);
+
 	if (rec_info & REC_INFO_MIN_REC_FLAG) {
 		/* When the rec is minimal rec in this level, we do
-		 in-place update for avoiding it move to other place. */
+		in-place update for avoiding it move to other place. */
+		page_zip_des_t* page_zip = buf_block_get_page_zip(block);
 
-		if (page_zip) {
+		if (UNIV_LIKELY_NULL(page_zip)) {
 			/* Check if there's enough space for in-place
 			update the zip page. */
 			if (!btr_cur_update_alloc_zip(
@@ -372,27 +297,23 @@ rtr_update_mbr_field(
 					rec, rec_offs_comp(offsets));
 			ut_ad(rec_info & REC_INFO_MIN_REC_FLAG);
 #endif /* UNIV_DEBUG */
-		}
-
-		if (!rtr_update_mbr_field_in_place(index, rec,
-						   offsets, mbr, mtr)) {
-			mem_heap_free(heap);
-			return(false);
-		}
-
-		if (page_zip) {
-			page_zip_write_rec(page_zip, rec, index, offsets, 0);
+			memcpy(rec, node_ptr->fields[0].data, DATA_MBR_LEN);
+			page_zip_write_rec(block, rec, index, offsets, 0, mtr);
+		} else {
+			mtr->memcpy<mtr_t::MAYBE_NOP>(*block, rec,
+						      node_ptr->fields[0].data,
+						      DATA_MBR_LEN);
 		}
 
 		if (cursor2) {
 			rec_offs* offsets2;
 
-			if (page_zip) {
+			if (UNIV_LIKELY_NULL(page_zip)) {
 				cursor2->page_cur.rec
 					= page_rec_get_nth(page, cur2_pos);
 			}
 			offsets2 = rec_get_offsets(btr_cur_get_rec(cursor2),
-						   index, NULL, 0,
+						   index, NULL, false,
 						   ULINT_UNDEFINED, &heap);
 			ut_ad(del_page_no == btr_node_ptr_get_child_page_no(
 							cursor2->page_cur.rec,
@@ -430,7 +351,7 @@ rtr_update_mbr_field(
 		ut_ad(old_rec != insert_rec);
 
 		page_cur_position(old_rec, block, &page_cur);
-		offsets2 = rec_get_offsets(old_rec, index, NULL, n_core,
+		offsets2 = rec_get_offsets(old_rec, index, NULL, is_leaf,
 					   ULINT_UNDEFINED, &heap);
 		page_cur_delete_rec(&page_cur, index, offsets2, mtr);
 
@@ -449,7 +370,7 @@ update_mbr:
 		if (!ins_suc) {
 			ut_ad(rec_info & REC_INFO_MIN_REC_FLAG);
 
-			btr_set_min_rec_mark(next_rec, mtr);
+			btr_set_min_rec_mark(next_rec, *block, mtr);
 		}
 
 		/* If there's more than 1 rec left in the page, delete
@@ -460,7 +381,7 @@ update_mbr:
 
 			cur2_rec = cursor2->page_cur.rec;
 			offsets2 = rec_get_offsets(cur2_rec, index, NULL,
-						   n_core,
+						   is_leaf,
 						   ULINT_UNDEFINED, &heap);
 
 			cur2_rec_info = rec_get_info_bits(cur2_rec,
@@ -471,7 +392,7 @@ update_mbr:
 				mark the new leftmost node pointer as
 				the predefined minimum record */
 				rec_t*	next_rec = page_rec_get_next(cur2_rec);
-				btr_set_min_rec_mark(next_rec, mtr);
+				btr_set_min_rec_mark(next_rec, *block, mtr);
 			}
 
 			ut_ad(del_page_no
@@ -520,7 +441,7 @@ update_mbr:
 		if (ins_suc) {
 			btr_cur_position(index, insert_rec, block, cursor);
 			offsets = rec_get_offsets(insert_rec,
-						  index, offsets, n_core,
+						  index, offsets, is_leaf,
 						  ULINT_UNDEFINED, &heap);
 		}
 
@@ -535,7 +456,7 @@ update_mbr:
 			cur2_rec = btr_cur_get_rec(cursor2);
 
 			offsets2 = rec_get_offsets(cur2_rec, index, NULL,
-						   n_core,
+						   is_leaf,
 						   ULINT_UNDEFINED, &heap);
 
 			/* If the cursor2 position is on a wrong rec, we
@@ -549,7 +470,7 @@ update_mbr:
 				while (!page_rec_is_supremum(cur2_rec)) {
 					offsets2 = rec_get_offsets(cur2_rec, index,
 								   NULL,
-								   n_core,
+								   is_leaf,
 								   ULINT_UNDEFINED,
 								   &heap);
 					cur2_pno = btr_node_ptr_get_child_page_no(
@@ -575,7 +496,7 @@ update_mbr:
 				mark the new leftmost node pointer as
 				the predefined minimum record */
 				rec_t*	next_rec = page_rec_get_next(cur2_rec);
-				btr_set_min_rec_mark(next_rec, mtr);
+				btr_set_min_rec_mark(next_rec, *block, mtr);
 			}
 
 			ut_ad(cur2_pno == del_page_no && cur2_rec != insert_rec);
@@ -631,12 +552,8 @@ rtr_adjust_upper_level(
 	rtr_mbr_t*	new_mbr,	/*!< in: MBR on the new page */
 	mtr_t*		mtr)		/*!< in: mtr */
 {
-	page_t*		page;
-	page_t*		new_page;
 	ulint		page_no;
 	ulint		new_page_no;
-	page_zip_des_t*	page_zip;
-	page_zip_des_t*	new_page_zip;
 	dict_index_t*	index = sea_cur->index;
 	btr_cur_t	cursor;
 	rec_offs*	offsets;
@@ -660,13 +577,9 @@ rtr_adjust_upper_level(
 	level = btr_page_get_level(buf_block_get_frame(block));
 	ut_ad(level == btr_page_get_level(buf_block_get_frame(new_block)));
 
-	page = buf_block_get_frame(block);
-	page_no = block->page.id.page_no();
-	page_zip = buf_block_get_page_zip(block);
+	page_no = block->page.id().page_no();
 
-	new_page = buf_block_get_frame(new_block);
-	new_page_no = new_block->page.id.page_no();
-	new_page_zip = buf_block_get_page_zip(new_block);
+	new_page_no = new_block->page.id().page_no();
 
 	/* Set new mbr for the old page on the upper level. */
 	/* Look up the index for the node pointer to page */
@@ -675,7 +588,8 @@ rtr_adjust_upper_level(
 
 	page_cursor = btr_cur_get_page_cur(&cursor);
 
-	rtr_update_mbr_field(&cursor, offsets, NULL, page, mbr, NULL, mtr);
+	rtr_update_mbr_field(&cursor, offsets, NULL, block->frame, mbr, NULL,
+			     mtr);
 
 	/* Already updated parent MBR, reset in our path */
 	if (sea_cur->rtr_info) {
@@ -689,7 +603,7 @@ rtr_adjust_upper_level(
 	/* Insert the node for the new page. */
 	node_ptr_upper = rtr_index_build_node_ptr(
 		index, new_mbr,
-		page_rec_get_next(page_get_infimum_rec(new_page)),
+		page_rec_get_next(page_get_infimum_rec(new_block->frame)),
 		new_page_no, heap);
 
 	ulint	up_match = 0;
@@ -739,36 +653,31 @@ rtr_adjust_upper_level(
 
 	lock_prdt_update_parent(block, new_block, &prdt, &new_prdt,
 				index->table->space_id,
-				page_cursor->block->page.id.page_no());
+				page_cursor->block->page.id().page_no());
 
 	mem_heap_free(heap);
 
 	ut_ad(block->zip_size() == index->table->space->zip_size());
 
-	const uint32_t next_page_no = btr_page_get_next(page);
+	const uint32_t next_page_no = btr_page_get_next(block->frame);
 
 	if (next_page_no != FIL_NULL) {
-		page_id_t	next_page_id(block->page.id.space(),
-					     next_page_no);
-
 		buf_block_t*	next_block = btr_block_get(
-			next_page_id, block->zip_size(), RW_X_LATCH,
-			index, mtr);
+			*index, next_page_no, RW_X_LATCH, false, mtr);
 #ifdef UNIV_BTR_DEBUG
-		ut_a(page_is_comp(next_block->frame) == page_is_comp(page));
+		ut_a(page_is_comp(next_block->frame)
+		     == page_is_comp(block->frame));
 		ut_a(btr_page_get_prev(next_block->frame)
-		     == block->page.id.page_no());
+		     == block->page.id().page_no());
 #endif /* UNIV_BTR_DEBUG */
 
-		btr_page_set_prev(buf_block_get_frame(next_block),
-				  buf_block_get_page_zip(next_block),
-				  new_page_no, mtr);
+		btr_page_set_prev(next_block, new_page_no, mtr);
 	}
 
-	btr_page_set_next(page, page_zip, new_page_no, mtr);
+	btr_page_set_next(block, new_page_no, mtr);
 
-	btr_page_set_prev(new_page, new_page_zip, page_no, mtr);
-	btr_page_set_next(new_page, new_page_zip, next_page_no, mtr);
+	btr_page_set_prev(new_block, page_no, mtr);
+	btr_page_set_next(new_block, next_page_no, mtr);
 }
 
 /*************************************************************//**
@@ -839,8 +748,7 @@ rtr_split_page_move_rec_list(
 	rec_move = static_cast<rtr_rec_move_t*>(mem_heap_alloc(
 			heap,
 			sizeof (*rec_move) * max_to_move));
-	const ulint n_core = page_is_leaf(page)
-		? index->n_core_fields : 0;
+	const bool is_leaf = page_is_leaf(page);
 
 	/* Insert the recs in group 2 to new page.  */
 	for (cur_split_node = node_array;
@@ -850,17 +758,14 @@ rtr_split_page_move_rec_list(
 				block, cur_split_node->key);
 
 			offsets = rec_get_offsets(cur_split_node->key,
-						  index, offsets, n_core,
+						  index, offsets, is_leaf,
 						  ULINT_UNDEFINED, &heap);
 
-			ut_ad(!n_core || cur_split_node->key != first_rec);
+			ut_ad(!is_leaf || cur_split_node->key != first_rec);
 
 			rec = page_cur_insert_rec_low(
-					page_cur_get_rec(&new_page_cursor),
-					index,
-					cur_split_node->key,
-					offsets,
-					mtr);
+				&new_page_cursor,
+				index, cur_split_node->key, offsets, mtr);
 
 			ut_a(rec);
 
@@ -888,7 +793,7 @@ rtr_split_page_move_rec_list(
 	same temp-table in parallel.
 	max_trx_id is ignored for temp tables because it not required
 	for MVCC. */
-	if (n_core && !index->table->is_temporary()) {
+	if (is_leaf && !index->table->is_temporary()) {
 		page_update_max_trx_id(new_block, NULL,
 				       page_get_max_trx_id(page),
 				       mtr);
@@ -897,7 +802,7 @@ rtr_split_page_move_rec_list(
 	if (new_page_zip) {
 		mtr_set_log_mode(mtr, log_mode);
 
-		if (!page_zip_compress(new_page_zip, new_page, index,
+		if (!page_zip_compress(new_block, index,
 				       page_zip_level, mtr)) {
 			ulint	ret_pos;
 
@@ -911,7 +816,8 @@ rtr_split_page_move_rec_list(
 			ret_pos == 0. */
 
 			if (UNIV_UNLIKELY
-			    (!page_zip_reorganize(new_block, index, mtr))) {
+			    (!page_zip_reorganize(new_block, index,
+						  page_zip_level, mtr))) {
 
 				if (UNIV_UNLIKELY
 				    (!page_zip_decompress(new_page_zip,
@@ -941,7 +847,7 @@ rtr_split_page_move_rec_list(
 					  block, &page_cursor);
 			offsets = rec_get_offsets(
 				page_cur_get_rec(&page_cursor), index,
-				offsets, n_core, ULINT_UNDEFINED,
+				offsets, is_leaf, ULINT_UNDEFINED,
 				&heap);
 			page_cur_delete_rec(&page_cursor,
 				index, offsets, mtr);
@@ -990,7 +896,6 @@ rtr_page_split_and_insert(
 	rtr_split_node_t*	cur_split_node;
 	rtr_split_node_t*	end_split_node;
 	double*			buf_pos;
-	ulint			page_level;
 	node_seq_t		current_ssn;
 	node_seq_t		next_ssn;
 	buf_block_t*		root_block;
@@ -1010,8 +915,8 @@ func_start:
 	mem_heap_empty(*heap);
 	*offsets = NULL;
 
-	ut_ad(mtr_memo_contains_flagged(mtr, dict_index_get_lock(cursor->index),
-					MTR_MEMO_X_LOCK | MTR_MEMO_SX_LOCK));
+	ut_ad(mtr->memo_contains_flagged(&cursor->index->lock, MTR_MEMO_X_LOCK
+					 | MTR_MEMO_SX_LOCK));
 	ut_ad(!dict_index_is_online_ddl(cursor->index)
 	      || (flags & BTR_CREATE_FLAG)
 	      || dict_index_is_clust(cursor->index));
@@ -1021,13 +926,12 @@ func_start:
 	block = btr_cur_get_block(cursor);
 	page = buf_block_get_frame(block);
 	page_zip = buf_block_get_page_zip(block);
-	page_level = btr_page_get_level(page);
 	current_ssn = page_get_ssn_id(page);
 
-	ut_ad(mtr_memo_contains(mtr, block, MTR_MEMO_PAGE_X_FIX));
+	ut_ad(mtr->memo_contains_flagged(block, MTR_MEMO_PAGE_X_FIX));
 	ut_ad(page_get_n_recs(page) >= 1);
 
-	page_no = block->page.id.page_no();
+	page_no = block->page.id().page_no();
 
 	if (!page_has_prev(page) && !page_is_leaf(page)) {
 		first_rec = page_rec_get_next(
@@ -1066,9 +970,19 @@ func_start:
 
 	/* Allocate a new page to the index */
 	hint_page_no = page_no + 1;
+	const uint16_t page_level = btr_page_get_level(page);
 	new_block = btr_page_alloc(cursor->index, hint_page_no, FSP_UP,
 				   page_level, mtr, mtr);
+	if (!new_block) {
+		return NULL;
+	}
+
 	new_page_zip = buf_block_get_page_zip(new_block);
+	if (page_level && UNIV_LIKELY_NULL(new_page_zip)) {
+		/* ROW_FORMAT=COMPRESSED non-leaf pages are not expected
+		to contain FIL_NULL in FIL_PAGE_PREV at this stage. */
+		memset_aligned<4>(new_block->frame + FIL_PAGE_PREV, 0, 4);
+	}
 	btr_page_create(new_block, new_page_zip, cursor->index,
 			page_level, mtr);
 
@@ -1105,7 +1019,7 @@ func_start:
 		as appropriate.  Deleting will always succeed. */
 		ut_a(new_page_zip);
 
-		page_zip_copy_recs(new_page_zip, new_page,
+		page_zip_copy_recs(new_block,
 				   page_zip, page, cursor->index, mtr);
 
 		page_cursor = btr_cur_get_page_cur(cursor);
@@ -1140,9 +1054,6 @@ func_start:
 		/* Update the lock table */
 		lock_rtr_move_rec_list(new_block, block, rec_move, moved);
 
-		const ulint n_core = page_level
-			? 0 : cursor->index->n_core_fields;
-
 		/* Delete recs in first group from the new page. */
 		for (cur_split_node = rtr_split_node_array;
 		     cur_split_node < end_split_node - 1; ++cur_split_node) {
@@ -1161,7 +1072,7 @@ func_start:
 
 				*offsets = rec_get_offsets(
 					page_cur_get_rec(page_cursor),
-					cursor->index, *offsets, n_core,
+					cursor->index, *offsets, !page_level,
 					ULINT_UNDEFINED, heap);
 
 				page_cur_delete_rec(page_cursor,
@@ -1178,7 +1089,7 @@ func_start:
 						  block, page_cursor);
 				*offsets = rec_get_offsets(
 					page_cur_get_rec(page_cursor),
-					cursor->index, *offsets, n_core,
+					cursor->index, *offsets, !page_level,
 					ULINT_UNDEFINED, heap);
 				page_cur_delete_rec(page_cursor,
 					cursor->index, *offsets, mtr);
@@ -1221,7 +1132,7 @@ func_start:
 	For compressed pages, page_cur_tuple_insert() will have
 	attempted this already. */
 	if (rec == NULL) {
-		if (!page_cur_get_page_zip(page_cursor)
+		if (!is_page_cur_get_page_zip(page_cursor)
 		    && btr_page_reorganize(page_cursor, cursor->index, mtr)) {
 			rec = page_cur_tuple_insert(page_cursor, tuple,
 						    cursor->index, offsets,
@@ -1259,6 +1170,15 @@ after_insert:
 	page_zip = buf_block_get_page_zip(root_block);
 	page_set_ssn_id(root_block, page_zip, next_ssn, mtr);
 
+	/* Insert fit on the page: update the free bits for the
+	left and right pages in the same mtr */
+
+	if (page_is_leaf(page)) {
+		ibuf_update_free_bits_for_two_pages_low(
+			block, new_block, mtr);
+	}
+
+
 	/* If the new res insert fail, we need to do another split
 	 again. */
 	if (!rec) {
@@ -1398,8 +1318,7 @@ rtr_page_copy_rec_list_end_no_locks(
 	rec_offs	offsets_2[REC_OFFS_NORMAL_SIZE];
 	rec_offs*	offsets2 = offsets_2;
 	ulint		moved = 0;
-	const ulint	n_core = page_is_leaf(new_page)
-		? index->n_core_fields : 0;
+	bool		is_leaf = page_is_leaf(new_page);
 
 	rec_offs_init(offsets_1);
 	rec_offs_init(offsets_2);
@@ -1428,14 +1347,14 @@ rtr_page_copy_rec_list_end_no_locks(
 			cur_rec = page_rec_get_next(cur_rec);
 		}
 
-		offsets1 = rec_get_offsets(cur1_rec, index, offsets1, n_core,
+		offsets1 = rec_get_offsets(cur1_rec, index, offsets1, is_leaf,
 					   ULINT_UNDEFINED, &heap);
 		while (!page_rec_is_supremum(cur_rec)) {
 			ulint		cur_matched_fields = 0;
 			int		cmp;
 
 			offsets2 = rec_get_offsets(cur_rec, index, offsets2,
-						   n_core,
+						   is_leaf,
 						   ULINT_UNDEFINED, &heap);
 			cmp = cmp_rec_rec(cur1_rec, cur_rec,
 					  offsets1, offsets2, index, false,
@@ -1447,7 +1366,7 @@ rtr_page_copy_rec_list_end_no_locks(
 				/* Skip small recs. */
 				page_cur_move_to_next(&page_cur);
 				cur_rec = page_cur_get_rec(&page_cur);
-			} else if (n_core) {
+			} else if (is_leaf) {
 				if (rec_get_deleted_flag(cur1_rec,
 					dict_table_is_comp(index->table))) {
 					goto next;
@@ -1455,8 +1374,8 @@ rtr_page_copy_rec_list_end_no_locks(
 					/* We have two identical leaf records,
 					skip copying the undeleted one, and
 					unmark deleted on the current page */
-					btr_rec_set_deleted_flag(
-						cur_rec, NULL, FALSE);
+					btr_rec_set_deleted<false>(
+						new_block, cur_rec, mtr);
 					goto next;
 				}
 			}
@@ -1470,15 +1389,15 @@ rtr_page_copy_rec_list_end_no_locks(
 
 		cur_rec = page_cur_get_rec(&page_cur);
 
-		offsets1 = rec_get_offsets(cur1_rec, index, offsets1, n_core,
+		offsets1 = rec_get_offsets(cur1_rec, index, offsets1, is_leaf,
 					   ULINT_UNDEFINED, &heap);
 
-		ins_rec = page_cur_insert_rec_low(cur_rec, index,
+		ins_rec = page_cur_insert_rec_low(&page_cur, index,
 						  cur1_rec, offsets1, mtr);
 		if (UNIV_UNLIKELY(!ins_rec)) {
-			fprintf(stderr, "page number %ld and %ld\n",
-				(long)new_block->page.id.page_no(),
-				(long)block->page.id.page_no());
+			fprintf(stderr, "page number %u and %u\n",
+				new_block->page.id().page_no(),
+				block->page.id().page_no());
 
 			ib::fatal() << "rec offset " << page_offset(rec)
 				<< ", cur1 offset "
@@ -1526,8 +1445,7 @@ rtr_page_copy_rec_list_start_no_locks(
 	rec_offs*	offsets2 = offsets_2;
 	page_cur_t	page_cur;
 	ulint		moved = 0;
-	const ulint	n_core = page_is_leaf(buf_block_get_frame(block))
-		? index->n_core_fields : 0;
+	bool		is_leaf = page_is_leaf(buf_block_get_frame(block));
 
 	rec_offs_init(offsets_1);
 	rec_offs_init(offsets_2);
@@ -1547,14 +1465,14 @@ rtr_page_copy_rec_list_start_no_locks(
 			cur_rec = page_rec_get_next(cur_rec);
 		}
 
-		offsets1 = rec_get_offsets(cur1_rec, index, offsets1, n_core,
+		offsets1 = rec_get_offsets(cur1_rec, index, offsets1, is_leaf,
 					   ULINT_UNDEFINED, &heap);
 
 		while (!page_rec_is_supremum(cur_rec)) {
 			ulint		cur_matched_fields = 0;
 
 			offsets2 = rec_get_offsets(cur_rec, index, offsets2,
-						   n_core,
+						   is_leaf,
 						   ULINT_UNDEFINED, &heap);
 			int cmp = cmp_rec_rec(cur1_rec, cur_rec,
 					      offsets1, offsets2, index, false,
@@ -1567,7 +1485,7 @@ rtr_page_copy_rec_list_start_no_locks(
 				/* Skip small recs. */
 				page_cur_move_to_next(&page_cur);
 				cur_rec = page_cur_get_rec(&page_cur);
-			} else if (n_core) {
+			} else if (is_leaf) {
 				if (rec_get_deleted_flag(
 					cur1_rec,
 					dict_table_is_comp(index->table))) {
@@ -1576,8 +1494,8 @@ rtr_page_copy_rec_list_start_no_locks(
 					/* We have two identical leaf records,
 					skip copying the undeleted one, and
 					unmark deleted on the current page */
-					btr_rec_set_deleted_flag(
-						cur_rec, NULL, FALSE);
+					btr_rec_set_deleted<false>(
+						new_block, cur_rec, mtr);
 					goto next;
 				}
 			}
@@ -1591,17 +1509,14 @@ rtr_page_copy_rec_list_start_no_locks(
 
 		cur_rec = page_cur_get_rec(&page_cur);
 
-		offsets1 = rec_get_offsets(cur1_rec, index, offsets1, n_core,
+		offsets1 = rec_get_offsets(cur1_rec, index, offsets1, is_leaf,
 					   ULINT_UNDEFINED, &heap);
 
-		ins_rec = page_cur_insert_rec_low(cur_rec, index,
+		ins_rec = page_cur_insert_rec_low(&page_cur, index,
 						  cur1_rec, offsets1, mtr);
 		if (UNIV_UNLIKELY(!ins_rec)) {
-			fprintf(stderr, "page number %ld and %ld\n",
-				(long)new_block->page.id.page_no(),
-				(long)block->page.id.page_no());
-
-			ib::fatal() << "rec offset " << page_offset(rec)
+			ib::fatal() << new_block->page.id()
+				<< "rec offset " << page_offset(rec)
 				<< ", cur1 offset "
 				<<  page_offset(page_cur_get_rec(&cur1))
 				<< ", cur_rec offset "
@@ -1738,14 +1653,14 @@ rtr_check_same_block(
 	mem_heap_t*	heap)	/*!< in: memory heap */
 
 {
-	ulint		page_no = childb->page.id.page_no();
+	ulint		page_no = childb->page.id().page_no();
 	rec_offs*	offsets;
 	rec_t*		rec = page_rec_get_next(page_get_infimum_rec(
 				buf_block_get_frame(parentb)));
 
 	while (!page_rec_is_supremum(rec)) {
 		offsets = rec_get_offsets(
-			rec, index, NULL, 0, ULINT_UNDEFINED, &heap);
+			rec, index, NULL, false, ULINT_UNDEFINED, &heap);
 
 		if (btr_node_ptr_get_child_page_no(rec, offsets) == page_no) {
 			btr_cur_position(index, rec, parentb, cursor);
@@ -1758,6 +1673,113 @@ rtr_check_same_block(
 	return(false);
 }
 
+/*************************************************************//**
+Calculates MBR_AREA(a+b) - MBR_AREA(a)
+Note: when 'a' and 'b' objects are far from each other,
+the area increase can be really big, so this function
+can return 'inf' as a result.
+Return the area increaed. */
+static double
+rtree_area_increase(
+	const uchar*	a,		/*!< in: original mbr. */
+	const uchar*	b,		/*!< in: new mbr. */
+	double*		ab_area)	/*!< out: increased area. */
+{
+	double		a_area = 1.0;
+	double		loc_ab_area = 1.0;
+	double		amin, amax, bmin, bmax;
+	double		data_round = 1.0;
+
+	static_assert(DATA_MBR_LEN == SPDIMS * 2 * sizeof(double),
+		      "compatibility");
+
+	for (auto i = SPDIMS; i--; ) {
+		double	area;
+
+		amin = mach_double_read(a);
+		bmin = mach_double_read(b);
+		amax = mach_double_read(a + sizeof(double));
+		bmax = mach_double_read(b + sizeof(double));
+
+		a += 2 * sizeof(double);
+		b += 2 * sizeof(double);
+
+		area = amax - amin;
+		if (area == 0) {
+			a_area *= LINE_MBR_WEIGHTS;
+		} else {
+			a_area *= area;
+		}
+
+		area = (double)std::max(amax, bmax) -
+		       (double)std::min(amin, bmin);
+		if (area == 0) {
+			loc_ab_area *= LINE_MBR_WEIGHTS;
+		} else {
+			loc_ab_area *= area;
+		}
+
+		/* Value of amax or bmin can be so large that small difference
+		are ignored. For example: 3.2884281489988079e+284 - 100 =
+		3.2884281489988079e+284. This results some area difference
+		are not detected */
+		if (loc_ab_area == a_area) {
+			if (bmin < amin || bmax > amax) {
+				data_round *= ((double)std::max(amax, bmax)
+					       - amax
+					       + (amin - (double)std::min(
+								amin, bmin)));
+			} else {
+				data_round *= area;
+			}
+		}
+	}
+
+	*ab_area = loc_ab_area;
+
+	if (loc_ab_area == a_area && data_round != 1.0) {
+		return(data_round);
+	}
+
+	return(loc_ab_area - a_area);
+}
+
+/** Calculates overlapping area
+@param[in]	a	mbr a
+@param[in]	b	mbr b
+@return overlapping area */
+static double rtree_area_overlapping(const byte *a, const byte *b)
+{
+	double	area = 1.0;
+	double	amin;
+	double	amax;
+	double	bmin;
+	double	bmax;
+
+	static_assert(DATA_MBR_LEN == SPDIMS * 2 * sizeof(double),
+		      "compatibility");
+
+	for (auto i = SPDIMS; i--; ) {
+		amin = mach_double_read(a);
+		bmin = mach_double_read(b);
+		amax = mach_double_read(a + sizeof(double));
+		bmax = mach_double_read(b + sizeof(double));
+		a += 2 * sizeof(double);
+		b += 2 * sizeof(double);
+
+		amin = std::max(amin, bmin);
+		amax = std::min(amax, bmax);
+
+		if (amin > amax) {
+			return(0);
+		} else {
+			area *= (amax - amin);
+		}
+	}
+
+	return(area);
+}
+
 /****************************************************************//**
 Calculate the area increased for a new record
 @return area increased */
@@ -1770,28 +1792,20 @@ rtr_rec_cal_increase(
 				dtuple in some of the common fields, or which
 				has an equal number or more fields than
 				dtuple */
-	const rec_offs*	offsets,/*!< in: array returned by rec_get_offsets() */
 	double*		area)	/*!< out: increased area */
 {
 	const dfield_t*	dtuple_field;
-	ulint		dtuple_f_len;
-	ulint		rec_f_len;
-	const byte*	rec_b_ptr;
-	double		ret = 0;
 
 	ut_ad(!page_rec_is_supremum(rec));
 	ut_ad(!page_rec_is_infimum(rec));
 
 	dtuple_field = dtuple_get_nth_field(dtuple, 0);
-	dtuple_f_len = dfield_get_len(dtuple_field);
+	ut_ad(dfield_get_len(dtuple_field) == DATA_MBR_LEN);
 
-	rec_b_ptr = rec_get_nth_field(rec, offsets, 0, &rec_f_len);
-	ret = rtree_area_increase(
-		rec_b_ptr,
-		static_cast<const byte*>(dfield_get_data(dtuple_field)),
-		static_cast<int>(dtuple_f_len), area);
-
-	return(ret);
+	return rtree_area_increase(rec,
+				   static_cast<const byte*>(
+					   dfield_get_data(dtuple_field)),
+				   area);
 }
 
 /** Estimates the number of rows in a given area.
@@ -1847,16 +1861,17 @@ rtr_estimate_n_rows_in_range(
 	index->set_modified(mtr);
 	mtr_s_lock_index(index, &mtr);
 
-	buf_block_t* block = btr_block_get(
-		page_id_t(index->table->space_id, index->page),
-		index->table->space->zip_size(),
-		RW_S_LATCH, index, &mtr);
+	buf_block_t* block = btr_root_block_get(index, RW_S_LATCH, &mtr);
+	if (!block) {
+err_exit:
+		mtr.commit();
+		return HA_POS_ERROR;
+	}
 	const page_t* page = buf_block_get_frame(block);
 	const unsigned n_recs = page_header_get_field(page, PAGE_N_RECS);
 
 	if (n_recs == 0) {
-		mtr.commit();
-		return(HA_POS_ERROR);
+		goto err_exit;
 	}
 
 	/* Scan records in root page and calculate area. */
@@ -1884,10 +1899,9 @@ rtr_estimate_n_rows_in_range(
 
 			case PAGE_CUR_WITHIN:
 			case PAGE_CUR_MBR_EQUAL:
-				if (rtree_key_cmp(
+				if (!rtree_key_cmp(
 					    PAGE_CUR_WITHIN, range_mbr_ptr,
-					    DATA_MBR_LEN, rec, DATA_MBR_LEN)
-				    == 0) {
+					    rec)) {
 					area += 1;
 				}
 
@@ -1901,14 +1915,14 @@ rtr_estimate_n_rows_in_range(
 			case PAGE_CUR_CONTAIN:
 			case PAGE_CUR_INTERSECT:
 				area += rtree_area_overlapping(
-					range_mbr_ptr, rec, DATA_MBR_LEN)
+					range_mbr_ptr, rec)
 					/ rec_area;
 				break;
 
 			case PAGE_CUR_DISJOINT:
 				area += 1;
 				area -= rtree_area_overlapping(
-					range_mbr_ptr, rec, DATA_MBR_LEN)
+					range_mbr_ptr, rec)
 					/ rec_area;
 				break;
 
@@ -1916,7 +1930,7 @@ rtr_estimate_n_rows_in_range(
 			case PAGE_CUR_MBR_EQUAL:
 				if (!rtree_key_cmp(
 					    PAGE_CUR_WITHIN, range_mbr_ptr,
-					    DATA_MBR_LEN, rec, DATA_MBR_LEN)) {
+					    rec)) {
 					area += range_area / rec_area;
 				}
 
@@ -1934,5 +1948,6 @@ rtr_estimate_n_rows_in_range(
 	}
 
 	area /= n_recs;
-	return ha_rows(dict_table_get_n_rows(index->table) * area);
+	return ha_rows(static_cast<double>(dict_table_get_n_rows(index->table))
+		       * area);
 }
