@@ -24,8 +24,7 @@ Full text search header file
 Created 2011/09/02 Sunny Bains
 ***********************************************************************/
 
-#ifndef fts0fts_h
-#define fts0fts_h
+#pragma once
 
 #include "data0type.h"
 #include "data0types.h"
@@ -349,6 +348,10 @@ public:
 	protected by fts_optimize_wq mutex */
 	bool		in_queue;
 
+	/** Whether the sync message exists in fts_optimize_wq;
+	protected by fts_optimize_wq mutex */
+	bool		sync_message;
+
 	/** Heap for fts_t allocation. */
 	mem_heap_t*	fts_heap;
 };
@@ -982,4 +985,12 @@ fts_trx_t*
 fts_trx_create(
 	trx_t*  trx);
 
-#endif /*!< fts0fts.h */
+/** Clear all fts resources when there is no internal DOC_ID
+and there are no new fts index to add.
+@param[in,out]  table   table  where fts is to be freed
+@param[in]      trx     transaction to drop all fts tables */
+void fts_clear_all(dict_table_t *table, trx_t *trx);
+
+/** Sync the table during commit phase
+@param[in]	table	table to be synced */
+void fts_sync_during_ddl(dict_table_t* table);
