@@ -92,9 +92,14 @@ struct Pool {
 			MEM_MAKE_ADDRESSABLE(&elem->m_type,
 					     sizeof elem->m_type);
 #endif
-			/* Declare the contents initialized;
-			we checked this in mem_free(). */
+#if defined HAVE_valgrind && !__has_feature(memory_sanitizer)
+			/* In Valgrind, we cannot cancel MEM_NOACCESS() without
+			changing the state of the V bits (which indicate
+			which bits are initialized).
+			We will declare the contents as initialized.
+			We did invoke MEM_CHECK_DEFINED() in mem_free(). */
 			MEM_MAKE_DEFINED(&elem->m_type, sizeof elem->m_type);
+#endif
 			Factory::destroy(&elem->m_type);
 		}
 
@@ -136,11 +141,14 @@ struct Pool {
 			MEM_MAKE_ADDRESSABLE(&elem->m_type,
 					     sizeof elem->m_type);
 # endif
-			/* Declare the memory initialized.
-			The trx_t that are released to the pool are
-			actually initialized; we checked that by
-			MEM_CHECK_DEFINED() in mem_free() below. */
+# if defined HAVE_valgrind && !__has_feature(memory_sanitizer)
+			/* In Valgrind, we cannot cancel MEM_NOACCESS() without
+			changing the state of the V bits (which indicate
+			which bits are initialized).
+			We will declare the contents as initialized.
+			We did invoke MEM_CHECK_DEFINED() in mem_free(). */
 			MEM_MAKE_DEFINED(&elem->m_type, sizeof elem->m_type);
+# endif
 		}
 #endif
 
