@@ -84,7 +84,6 @@ row_undo_ins_remove_clust_rec(
 		online = false;
 	} else {
 		index->set_modified(mtr);
-		ut_ad(lock_table_has_locks(index->table));
 		online = dict_index_is_online_ddl(index);
 		if (online) {
 			ut_ad(node->rec_type == TRX_UNDO_INSERT_REC);
@@ -131,8 +130,7 @@ row_undo_ins_remove_clust_rec(
 			      == RW_X_LATCH);
 			ut_ad(node->rec_type == TRX_UNDO_INSERT_REC);
 
-			dict_drop_index_tree(rec, &node->pcur, node->trx,
-					     &mtr);
+			dict_drop_index_tree(&node->pcur, node->trx, &mtr);
 			mtr.commit();
 
 			mtr.start();
@@ -208,28 +206,7 @@ func_exit:
 	if (err == DB_SUCCESS && node->rec_type == TRX_UNDO_INSERT_METADATA) {
 		/* When rolling back the very first instant ADD COLUMN
 		operation, reset the root page to the basic state. */
-		ut_ad(!index->table->is_temporary());
-		if (page_t* root = btr_root_get(index, &mtr)) {
-			byte* page_type = root + FIL_PAGE_TYPE;
-			ut_ad(mach_read_from_2(page_type)
-			      == FIL_PAGE_TYPE_INSTANT
-			      || mach_read_from_2(page_type)
-			      == FIL_PAGE_INDEX);
-			mlog_write_ulint(page_type, FIL_PAGE_INDEX,
-					 MLOG_2BYTES, &mtr);
-			byte* instant = PAGE_INSTANT + PAGE_HEADER + root;
-			mlog_write_ulint(instant,
-					 page_ptr_get_direction(instant + 1),
-					 MLOG_2BYTES, &mtr);
-			rec_t* infimum = page_get_infimum_rec(root);
-			rec_t* supremum = page_get_supremum_rec(root);
-			static const byte str[8 + 8] = "supremuminfimum";
-			if (memcmp(infimum, str + 8, 8)
-			    || memcmp(supremum, str, 8)) {
-				mlog_write_string(infimum, str + 8, 8, &mtr);
-				mlog_write_string(supremum, str, 8, &mtr);
-			}
-		}
+		btr_reset_instant(*index, true, &mtr);
 	}
 
 	btr_pcur_commit_specify_mtr(&node->pcur, &mtr);
@@ -402,7 +379,7 @@ static bool row_undo_ins_parse_undo_rec(undo_node_t* node, bool dict_locked)
 
 	switch (node->rec_type) {
 	default:
-		ut_ad(!"wrong undo record type");
+		ut_ad("wrong undo record type" == 0);
 		goto close_table;
 	case TRX_UNDO_INSERT_METADATA:
 	case TRX_UNDO_INSERT_REC:
@@ -448,13 +425,6 @@ close_table:
 					node->heap);
 			} else {
 				node->ref = &trx_undo_metadata;
-				if (!row_undo_search_clust_to_pcur(node)) {
-					/* An error probably occurred during
-					an insert into the clustered index,
-					after we wrote the undo log record. */
-					goto close_table;
-				}
-				return true;
 			}
 
 			if (!row_undo_search_clust_to_pcur(node)) {
@@ -566,7 +536,7 @@ row_undo_ins(
 
 	switch (node->rec_type) {
 	default:
-		ut_ad(!"wrong undo record type");
+		ut_ad("wrong undo record type" == 0);
 		/* fall through */
 	case TRX_UNDO_INSERT_REC:
 		/* Skip the clustered index (the first index) */
@@ -596,7 +566,7 @@ row_undo_ins(
 		}
 
 		if (err == DB_SUCCESS && node->table->stat_initialized) {
-			/* Not protected by dict_sys.mutex for
+			/* Not protected by dict_table_stats_lock() for
 			performance reasons, we would rather get garbage
 			in stat_n_rows (which is just an estimate anyway)
 			than protecting the following code with a latch. */
