@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 2007, 2015, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, 2021, MariaDB Corporation.
+Copyright (c) 2017, 2020, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -141,15 +141,18 @@ struct i_s_table_cache_t {
 struct trx_i_s_cache_t {
 	rw_lock_t	rw_lock;	/*!< read-write lock protecting
 					the rest of this structure */
-	Atomic_relaxed<ulonglong> last_read;
-					/*!< last time the cache was read;
+	ulonglong	last_read;	/*!< last time the cache was read;
 					measured in nanoseconds */
+	ib_mutex_t		last_read_mutex;/*!< mutex protecting the
+					last_read member - it is updated
+					inside a shared lock of the
+					rw_lock member */
 	i_s_table_cache_t innodb_trx;	/*!< innodb_trx table */
 	i_s_table_cache_t innodb_locks;	/*!< innodb_locks table */
 	i_s_table_cache_t innodb_lock_waits;/*!< innodb_lock_waits table */
 /** the hash table size is LOCKS_HASH_CELLS_NUM * sizeof(void*) bytes */
 #define LOCKS_HASH_CELLS_NUM		10000
-	hash_table_t*	locks_hash;	/*!< hash table used to eliminate
+	hash_table_t	locks_hash;	/*!< hash table used to eliminate
 					duplicate entries in the
 					innodb_locks table */
 /** Initial size of the cache storage */
@@ -159,7 +162,7 @@ struct trx_i_s_cache_t {
 	ha_storage_t*	storage;	/*!< storage for external volatile
 					data that may become unavailable
 					when we release
-					lock_sys.mutex or trx_sys.mutex */
+					lock_sys.mutex */
 	ulint		mem_allocd;	/*!< the amount of memory
 					allocated with mem_alloc*() */
 	bool		is_truncated;	/*!< this is true if the memory
@@ -176,31 +179,13 @@ INFORMATION SCHEMA tables is fetched and later retrieved by the C++
 code in handler/i_s.cc. */
 trx_i_s_cache_t*	trx_i_s_cache = &trx_i_s_cache_static;
 
-/*******************************************************************//**
-For a record lock that is in waiting state retrieves the only bit that
-is set, for a table lock returns ULINT_UNDEFINED.
-@return record number within the heap */
-static
-ulint
-wait_lock_get_heap_no(
-/*==================*/
-	const lock_t*	lock)	/*!< in: lock */
+/** @return the heap number of a record lock
+@retval 0xFFFF for table locks */
+static uint16_t wait_lock_get_heap_no(const lock_t *lock)
 {
-	ulint	ret;
-
-	switch (lock_get_type(lock)) {
-	case LOCK_REC:
-		ret = lock_rec_find_set_bit(lock);
-		ut_a(ret != ULINT_UNDEFINED);
-		break;
-	case LOCK_TABLE:
-		ret = ULINT_UNDEFINED;
-		break;
-	default:
-		ut_error;
-	}
-
-	return(ret);
+  return lock_get_type(lock) == LOCK_REC
+    ? static_cast<uint16_t>(lock_rec_find_set_bit(lock))
+    : uint16_t{0xFFFF};
 }
 
 /*******************************************************************//**
@@ -402,25 +387,20 @@ i_s_locks_row_validate(
 /*===================*/
 	const i_s_locks_row_t*	row)	/*!< in: row to validate */
 {
-	ut_ad(row->lock_mode != NULL);
-	ut_ad(row->lock_type != NULL);
+	ut_ad(row->lock_mode);
 	ut_ad(row->lock_table != NULL);
 	ut_ad(row->lock_table_id != 0);
 
-	if (row->lock_space == ULINT_UNDEFINED) {
+	if (!row->lock_index) {
 		/* table lock */
-		ut_ad(!strcmp("TABLE", row->lock_type));
-		ut_ad(row->lock_index == NULL);
-		ut_ad(row->lock_data == NULL);
-		ut_ad(row->lock_page == ULINT_UNDEFINED);
-		ut_ad(row->lock_rec == ULINT_UNDEFINED);
+		ut_ad(!row->lock_data);
+		ut_ad(!row->lock_space);
+		ut_ad(!row->lock_page);
+		ut_ad(!row->lock_rec);
 	} else {
 		/* record lock */
-		ut_ad(!strcmp("RECORD", row->lock_type));
-		ut_ad(row->lock_index != NULL);
 		/* row->lock_data == NULL if buf_page_try_get() == NULL */
-		ut_ad(row->lock_page != ULINT_UNDEFINED);
-		ut_ad(row->lock_rec != ULINT_UNDEFINED);
+		ut_ad(row->lock_page);
 	}
 
 	return(TRUE);
@@ -501,21 +481,7 @@ fill_trx_row(
 	}
 
 thd_done:
-	s = trx->op_info;
-
-	if (s != NULL && s[0] != '\0') {
-
-		TRX_I_S_STRING_COPY(s, row->trx_operation_state,
-				    TRX_I_S_TRX_OP_STATE_MAX_LEN, cache);
-
-		if (row->trx_operation_state == NULL) {
-
-			return(FALSE);
-		}
-	} else {
-
-		row->trx_operation_state = NULL;
-	}
+	row->trx_operation_state = trx->op_info;
 
 	row->trx_tables_in_use = trx->n_mysql_tables_in_use;
 
@@ -533,25 +499,7 @@ thd_done:
 
 	row->trx_rows_modified = trx->undo_no;
 
-	row->trx_concurrency_tickets = trx->n_tickets_to_enter_innodb;
-
-	switch (trx->isolation_level) {
-	case TRX_ISO_READ_UNCOMMITTED:
-		row->trx_isolation_level = "READ UNCOMMITTED";
-		break;
-	case TRX_ISO_READ_COMMITTED:
-		row->trx_isolation_level = "READ COMMITTED";
-		break;
-	case TRX_ISO_REPEATABLE_READ:
-		row->trx_isolation_level = "REPEATABLE READ";
-		break;
-	case TRX_ISO_SERIALIZABLE:
-		row->trx_isolation_level = "SERIALIZABLE";
-		break;
-	/* Should not happen as TRX_ISO_READ_COMMITTED is default */
-	default:
-		row->trx_isolation_level = "UNKNOWN";
-	}
+	row->trx_isolation_level = trx->isolation_level;
 
 	row->trx_unique_checks = (ibool) trx->check_unique_secondary;
 
@@ -575,7 +523,7 @@ thd_done:
 
 	row->trx_is_read_only = trx->read_only;
 
-	row->trx_is_autocommit_non_locking = trx->is_autocommit_non_locking();
+	row->trx_is_autocommit_non_locking = trx_is_autocommit_non_locking(trx);
 
 	return(TRUE);
 }
@@ -683,8 +631,8 @@ fill_lock_data(
 
 	mtr_start(&mtr);
 
-	block = buf_page_try_get(page_id_t(lock_rec_get_space_id(lock),
-					   lock_rec_get_page_no(lock)),
+	block = buf_page_try_get(page_id_t(lock->un_member.rec_lock.space,
+					   lock->un_member.rec_lock.page_no),
 				 &mtr);
 
 	if (block == NULL) {
@@ -710,8 +658,7 @@ fill_lock_data(
 	ut_a(n_fields > 0);
 
 	heap = NULL;
-	offsets = rec_get_offsets(rec, index, offsets, index->n_core_fields,
-				  n_fields, &heap);
+	offsets = rec_get_offsets(rec, index, offsets, true, n_fields, &heap);
 
 	/* format and store the data */
 
@@ -749,22 +696,42 @@ fill_lock_data(
 /*******************************************************************//**
 Fills i_s_locks_row_t object. Returns its first argument.
 If memory can not be allocated then FALSE is returned.
-@return FALSE if allocation fails */
-static
-ibool
-fill_locks_row(
-/*===========*/
+@return false if allocation fails */
+static bool fill_locks_row(
 	i_s_locks_row_t* row,	/*!< out: result object that's filled */
 	const lock_t*	lock,	/*!< in: lock to get data from */
-	ulint		heap_no,/*!< in: lock's record number
-				or ULINT_UNDEFINED if the lock
+	uint16_t	heap_no,/*!< in: lock's record number
+				or 0 if the lock
 				is a table lock */
 	trx_i_s_cache_t* cache)	/*!< in/out: cache into which to copy
 				volatile strings */
 {
-	row->lock_trx_id = lock_get_trx_id(lock);
-	row->lock_mode = lock_get_mode_str(lock);
-	row->lock_type = lock_get_type_str(lock);
+	row->lock_trx_id = lock->trx->id;
+	const auto lock_type = lock_get_type(lock);
+	ut_ad(lock_type == LOCK_REC || lock_type == LOCK_TABLE);
+
+	const bool is_gap_lock = lock_type == LOCK_REC
+		&& (lock->type_mode & LOCK_GAP);
+	switch (lock->type_mode & LOCK_MODE_MASK) {
+	case LOCK_S:
+		row->lock_mode = uint8_t(1 + is_gap_lock);
+		break;
+	case LOCK_X:
+		row->lock_mode = uint8_t(3 + is_gap_lock);
+		break;
+	case LOCK_IS:
+		row->lock_mode = uint8_t(5 + is_gap_lock);
+		break;
+	case LOCK_IX:
+		row->lock_mode = uint8_t(7 + is_gap_lock);
+		break;
+	case LOCK_AUTO_INC:
+		row->lock_mode = 9;
+		break;
+	default:
+		ut_ad("unknown lock mode" == 0);
+		row->lock_mode = 0;
+	}
 
 	row->lock_table = ha_storage_put_str_memlim(
 		cache->storage, lock_get_table_name(lock).m_name,
@@ -773,11 +740,10 @@ fill_locks_row(
 	/* memory could not be allocated */
 	if (row->lock_table == NULL) {
 
-		return(FALSE);
+		return false;
 	}
 
-	switch (lock_get_type(lock)) {
-	case LOCK_REC:
+	if (lock_type == LOCK_REC) {
 		row->lock_index = ha_storage_put_str_memlim(
 			cache->storage, lock_rec_get_index_name(lock),
 			MAX_ALLOWED_FOR_STORAGE(cache));
@@ -785,32 +751,26 @@ fill_locks_row(
 		/* memory could not be allocated */
 		if (row->lock_index == NULL) {
 
-			return(FALSE);
+			return false;
 		}
 
-		row->lock_space = lock_rec_get_space_id(lock);
-		row->lock_page = lock_rec_get_page_no(lock);
+		row->lock_space = lock->un_member.rec_lock.space;
+		row->lock_page = lock->un_member.rec_lock.page_no;
 		row->lock_rec = heap_no;
 
 		if (!fill_lock_data(&row->lock_data, lock, heap_no, cache)) {
 
 			/* memory could not be allocated */
-			return(FALSE);
+			return false;
 		}
-
-		break;
-	case LOCK_TABLE:
+	} else {
 		row->lock_index = NULL;
 
-		row->lock_space = ULINT_UNDEFINED;
-		row->lock_page = ULINT_UNDEFINED;
-		row->lock_rec = ULINT_UNDEFINED;
+		row->lock_space = 0;
+		row->lock_page = 0;
+		row->lock_rec = 0;
 
 		row->lock_data = NULL;
-
-		break;
-	default:
-		ut_error;
 	}
 
 	row->lock_table_id = lock_get_table_id(lock);
@@ -818,7 +778,7 @@ fill_locks_row(
 	row->hash_chain.value = row;
 	ut_ad(i_s_locks_row_validate(row));
 
-	return(TRUE);
+	return true;
 }
 
 /*******************************************************************//**
@@ -858,7 +818,7 @@ fold_lock(
 /*======*/
 	const lock_t*	lock,	/*!< in: lock object to fold */
 	ulint		heap_no)/*!< in: lock's record number
-				or ULINT_UNDEFINED if the lock
+				or 0xFFFF if the lock
 				is a table lock */
 {
 #ifdef TEST_LOCK_FOLD_ALWAYS_DIFFERENT
@@ -870,13 +830,13 @@ fold_lock(
 
 	switch (lock_get_type(lock)) {
 	case LOCK_REC:
-		ut_a(heap_no != ULINT_UNDEFINED);
+		ut_a(heap_no != 0xFFFF);
 
-		ret = ut_fold_ulint_pair((ulint) lock_get_trx_id(lock),
-					 lock_rec_get_space_id(lock));
+		ret = ut_fold_ulint_pair((ulint) lock->trx->id,
+					 lock->un_member.rec_lock.space);
 
 		ret = ut_fold_ulint_pair(ret,
-					 lock_rec_get_page_no(lock));
+					 lock->un_member.rec_lock.page_no);
 
 		ret = ut_fold_ulint_pair(ret, heap_no);
 
@@ -885,7 +845,7 @@ fold_lock(
 		/* this check is actually not necessary for continuing
 		correct operation, but something must have gone wrong if
 		it fails. */
-		ut_a(heap_no == ULINT_UNDEFINED);
+		ut_a(heap_no == 0xFFFF);
 
 		ret = (ulint) lock_get_table_id(lock);
 
@@ -908,7 +868,7 @@ locks_row_eq_lock(
 	const i_s_locks_row_t*	row,	/*!< in: innodb_locks row */
 	const lock_t*		lock,	/*!< in: lock object */
 	ulint			heap_no)/*!< in: lock's record number
-					or ULINT_UNDEFINED if the lock
+					or 0xFFFF if the lock
 					is a table lock */
 {
 	ut_ad(i_s_locks_row_validate(row));
@@ -917,20 +877,20 @@ locks_row_eq_lock(
 #else
 	switch (lock_get_type(lock)) {
 	case LOCK_REC:
-		ut_a(heap_no != ULINT_UNDEFINED);
+		ut_a(heap_no != 0xFFFF);
 
-		return(row->lock_trx_id == lock_get_trx_id(lock)
-		       && row->lock_space == lock_rec_get_space_id(lock)
-		       && row->lock_page == lock_rec_get_page_no(lock)
+		return(row->lock_trx_id == lock->trx->id
+		       && row->lock_space == lock->un_member.rec_lock.space
+		       && row->lock_page == lock->un_member.rec_lock.page_no
 		       && row->lock_rec == heap_no);
 
 	case LOCK_TABLE:
 		/* this check is actually not necessary for continuing
 		correct operation, but something must have gone wrong if
 		it fails. */
-		ut_a(heap_no == ULINT_UNDEFINED);
+		ut_a(heap_no == 0xFFFF);
 
-		return(row->lock_trx_id == lock_get_trx_id(lock)
+		return(row->lock_trx_id == lock->trx->id
 		       && row->lock_table_id == lock_get_table_id(lock));
 
 	default:
@@ -951,8 +911,8 @@ search_innodb_locks(
 /*================*/
 	trx_i_s_cache_t*	cache,	/*!< in: cache */
 	const lock_t*		lock,	/*!< in: lock to search for */
-	ulint			heap_no)/*!< in: lock's record number
-					or ULINT_UNDEFINED if the lock
+	uint16_t		heap_no)/*!< in: lock's record number
+					or 0xFFFF if the lock
 					is a table lock */
 {
 	i_s_hash_chain_t*	hash_chain;
@@ -961,7 +921,7 @@ search_innodb_locks(
 		/* hash_chain->"next" */
 		next,
 		/* the hash table */
-		cache->locks_hash,
+		&cache->locks_hash,
 		/* fold */
 		fold_lock(lock, heap_no),
 		/* the type of the next variable */
@@ -994,8 +954,8 @@ add_lock_to_cache(
 /*==============*/
 	trx_i_s_cache_t*	cache,	/*!< in/out: cache */
 	const lock_t*		lock,	/*!< in: the element to add */
-	ulint			heap_no)/*!< in: lock's record number
-					or ULINT_UNDEFINED if the lock
+	uint16_t		heap_no)/*!< in: lock's record number
+					or 0 if the lock
 					is a table lock */
 {
 	i_s_locks_row_t*	dst_row;
@@ -1037,7 +997,7 @@ add_lock_to_cache(
 		/* hash_chain->"next" */
 		next,
 		/* the hash table */
-		cache->locks_hash,
+		&cache->locks_hash,
 		/* fold */
 		fold_lock(lock, heap_no),
 		/* add this data to the hash */
@@ -1109,13 +1069,12 @@ add_trx_relevant_locks_to_cache(
 	if (trx->lock.que_state == TRX_QUE_LOCK_WAIT) {
 
 		const lock_t*		curr_lock;
-		ulint			wait_lock_heap_no;
 		i_s_locks_row_t*	blocking_lock_row;
 		lock_queue_iterator_t	iter;
 
 		ut_a(trx->lock.wait_lock != NULL);
 
-		wait_lock_heap_no
+		uint16_t wait_lock_heap_no
 			= wait_lock_get_heap_no(trx->lock.wait_lock);
 
 		/* add the requested lock */
@@ -1188,7 +1147,8 @@ Checks if the cache can safely be updated.
 @return whether the cache can be updated */
 static bool can_cache_be_updated(trx_i_s_cache_t* cache)
 {
-	/* cache->last_read is only updated when a shared rw lock on the
+	/* Here we read cache->last_read without acquiring its mutex
+	because last_read is only updated when a shared rw lock on the
 	whole cache is being held (see trx_i_s_cache_end_read()) and
 	we are currently holding an exclusive rw lock on the cache.
 	So it is not possible for last_read to be updated while we are
@@ -1212,7 +1172,7 @@ trx_i_s_cache_clear(
 	cache->innodb_locks.rows_used = 0;
 	cache->innodb_lock_waits.rows_used = 0;
 
-	hash_table_clear(cache->locks_hash);
+	cache->locks_hash.clear();
 
 	ha_storage_empty(&cache->storage);
 }
@@ -1229,24 +1189,7 @@ static void fetch_data_into_cache_low(trx_i_s_cache_t *cache, const trx_t *trx)
 {
   i_s_locks_row_t *requested_lock_row;
 
-#ifdef UNIV_DEBUG
-  {
-    const auto state= trx->state;
-
-    if (trx->is_autocommit_non_locking())
-    {
-      ut_ad(trx->read_only);
-      ut_ad(!trx->is_recovered);
-      ut_ad(trx->mysql_thd);
-      ut_ad(state == TRX_STATE_NOT_STARTED || state == TRX_STATE_ACTIVE);
-    }
-    else
-      ut_ad(state == TRX_STATE_ACTIVE ||
-            state == TRX_STATE_PREPARED ||
-            state == TRX_STATE_PREPARED_RECOVERED ||
-            state == TRX_STATE_COMMITTED_IN_MEMORY);
-  }
-#endif /* UNIV_DEBUG */
+  assert_trx_nonlocking_or_in_list(trx);
 
   if (add_trx_relevant_locks_to_cache(cache, trx, &requested_lock_row))
   {
@@ -1275,22 +1218,13 @@ static void fetch_data_into_cache(trx_i_s_cache_t *cache)
   trx_i_s_cache_clear(cache);
 
   /* Capture the state of transactions */
-  mutex_enter(&trx_sys.mutex);
-  for (trx_t *trx= UT_LIST_GET_FIRST(trx_sys.trx_list);
-       trx != NULL;
-       trx= UT_LIST_GET_NEXT(trx_list, trx))
-  {
-    if (trx->state != TRX_STATE_NOT_STARTED && trx != purge_sys.query->trx)
+  trx_sys.trx_list.for_each([cache](const trx_t &trx) {
+    if (!cache->is_truncated && trx_is_started(&trx) &&
+        &trx != purge_sys.query->trx)
     {
-      mutex_enter(&trx->mutex);
-      if (trx->state != TRX_STATE_NOT_STARTED)
-        fetch_data_into_cache_low(cache, trx);
-      mutex_exit(&trx->mutex);
-      if (cache->is_truncated)
-        break;
-     }
-  }
-  mutex_exit(&trx_sys.mutex);
+      fetch_data_into_cache_low(cache, &trx);
+    }
+  });
   cache->is_truncated= false;
 }
 
@@ -1346,6 +1280,8 @@ trx_i_s_cache_init(
 	release lock mutex
 	release trx_i_s_cache_t::rw_lock
 	acquire trx_i_s_cache_t::rw_lock, S
+	acquire trx_i_s_cache_t::last_read_mutex
+	release trx_i_s_cache_t::last_read_mutex
 	release trx_i_s_cache_t::rw_lock */
 
 	rw_lock_create(trx_i_s_cache_lock_key, &cache->rw_lock,
@@ -1353,12 +1289,14 @@ trx_i_s_cache_init(
 
 	cache->last_read = 0;
 
+	mutex_create(LATCH_ID_CACHE_LAST_READ, &cache->last_read_mutex);
+
 	table_cache_init(&cache->innodb_trx, sizeof(i_s_trx_row_t));
 	table_cache_init(&cache->innodb_locks, sizeof(i_s_locks_row_t));
 	table_cache_init(&cache->innodb_lock_waits,
 			 sizeof(i_s_lock_waits_row_t));
 
-	cache->locks_hash = hash_create(LOCKS_HASH_CELLS_NUM);
+	cache->locks_hash.create(LOCKS_HASH_CELLS_NUM);
 
 	cache->storage = ha_storage_create(CACHE_STORAGE_INITIAL_SIZE,
 					   CACHE_STORAGE_HASH_CELLS);
@@ -1376,8 +1314,9 @@ trx_i_s_cache_free(
 	trx_i_s_cache_t*	cache)	/*!< in, own: cache to free */
 {
 	rw_lock_free(&cache->rw_lock);
+	mutex_free(&cache->last_read_mutex);
 
-	hash_table_free(cache->locks_hash);
+	cache->locks_hash.free();
 	ha_storage_free(cache->storage);
 	table_cache_free(&cache->innodb_trx);
 	table_cache_free(&cache->innodb_locks);
@@ -1401,7 +1340,14 @@ trx_i_s_cache_end_read(
 /*===================*/
 	trx_i_s_cache_t*	cache)	/*!< in: cache */
 {
-	cache->last_read = my_interval_timer();
+	ut_ad(rw_lock_own(&cache->rw_lock, RW_LOCK_S));
+
+	/* update cache last read time */
+	const ulonglong now = my_interval_timer();
+	mutex_enter(&cache->last_read_mutex);
+	cache->last_read = now;
+	mutex_exit(&cache->last_read_mutex);
+
 	rw_lock_s_unlock(&cache->rw_lock);
 }
 
@@ -1526,11 +1472,11 @@ trx_i_s_create_lock_id(
 
 	/* please adjust TRX_I_S_LOCK_ID_MAX_LEN if you change this */
 
-	if (row->lock_space != ULINT_UNDEFINED) {
+	if (row->lock_index) {
 		/* record lock */
 		res_len = snprintf(lock_id, lock_id_size,
 				   TRX_ID_FMT
-				   ":" ULINTPF ":" ULINTPF ":" ULINTPF,
+				   ":%u:%u:%u",
 				   row->lock_trx_id, row->lock_space,
 				   row->lock_page, row->lock_rec);
 	} else {
