@@ -254,7 +254,7 @@ int table2myisam(TABLE *table_arg, MI_KEYDEF **keydef_out,
   TABLE_SHARE *share= table_arg->s;
   uint options= share->db_options_in_use;
   DBUG_ENTER("table2myisam");
-  if (!(my_multi_malloc(MYF(MY_WME),
+  if (!(my_multi_malloc(PSI_INSTRUMENT_ME, MYF(MY_WME),
           recinfo_out, (share->fields * 2 + 2) * sizeof(MI_COLUMNDEF),
           keydef_out, share->keys * sizeof(MI_KEYDEF),
           &keyseg,
@@ -688,8 +688,6 @@ my_bool mi_killed_in_mariadb(MI_INFO *info)
 
 static int compute_vcols(MI_INFO *info, uchar *record, int keynum)
 {
-  /* This mutex is needed for parallel repair */
-  mysql_mutex_lock(&info->s->intern_lock);
   TABLE *table= (TABLE*)(info->external_ref);
   table->move_fields(table->field, record, table->field[0]->record_ptr());
   if (keynum == -1) // update all vcols
@@ -697,7 +695,6 @@ static int compute_vcols(MI_INFO *info, uchar *record, int keynum)
     int error= table->update_virtual_fields(table->file, VCOL_UPDATE_FOR_READ);
     if (table->update_virtual_fields(table->file, VCOL_UPDATE_INDEXED))
       error= 1;
-    mysql_mutex_unlock(&info->s->intern_lock);
     return error;
   }
   // update only one key
@@ -706,10 +703,9 @@ static int compute_vcols(MI_INFO *info, uchar *record, int keynum)
   for (; kp < end; kp++)
   {
     Field *f= table->field[kp->fieldnr - 1];
-    if (f->vcol_info && !f->vcol_info->stored_in_db)
+    if (f->vcol_info)
       table->update_virtual_field(f);
   }
-  mysql_mutex_unlock(&info->s->intern_lock);
   return 0;
 }
 
@@ -726,7 +722,7 @@ ha_myisam::ha_myisam(handlerton *hton, TABLE_SHARE *table_arg)
                   HA_CAN_INSERT_DELAYED | HA_CAN_BIT_FIELD | HA_CAN_RTREEKEYS |
                   HA_HAS_RECORDS | HA_STATS_RECORDS_IS_EXACT | HA_CAN_REPAIR |
                   HA_CAN_TABLES_WITHOUT_ROLLBACK),
-   can_enable_indexes(0)
+   can_enable_indexes(1)
 {}
 
 handler *ha_myisam::clone(const char *name __attribute__((unused)),
@@ -960,7 +956,7 @@ void ha_myisam::setup_vcols_for_repair(HA_CHECK *param)
   if (!table->vfield)
     return;
 
-  if (file->s->base.reclength == file->s->vreclength)
+  if  (file->s->base.reclength == file->s->vreclength)
   {
     bool indexed_vcols= false;
     ulong new_vreclength= file->s->vreclength;
@@ -968,8 +964,7 @@ void ha_myisam::setup_vcols_for_repair(HA_CHECK *param)
     {
       if (!(*vf)->stored_in_db())
       {
-        uint vf_end= ((*vf)->offset(table->record[0]) +
-                      (*vf)->pack_length_in_rec());
+        uint vf_end= (*vf)->offset(table->record[0]) + (*vf)->pack_length_in_rec();
         set_if_bigger(new_vreclength, vf_end);
         indexed_vcols|= ((*vf)->flags & PART_KEY_FLAG) != 0;
       }
@@ -987,8 +982,7 @@ void ha_myisam::restore_vcos_after_repair()
 {
   if (file->s->base.reclength < file->s->vreclength)
   {
-    table->move_fields(table->field, table->record[0],
-                       table->field[0]->record_ptr());
+    table->move_fields(table->field, table->record[0], table->field[0]->record_ptr());
     table->default_column_bitmaps();
   }
 }
@@ -1729,12 +1723,11 @@ void ha_myisam::start_bulk_insert(ha_rows rows, uint flags)
   THD *thd= table->in_use;
   ulong size= MY_MIN(thd->variables.read_buff_size,
                      (ulong) (table->s->avg_row_length*rows));
-  bool index_disabled= 0;
   DBUG_PRINT("info",("start_bulk_insert: rows %lu size %lu",
                      (ulong) rows, size));
 
   /* don't enable row cache if too few rows */
-  if (! rows || (rows > MI_MIN_ROWS_TO_USE_WRITE_CACHE))
+  if ((!rows || rows > MI_MIN_ROWS_TO_USE_WRITE_CACHE) && !has_long_unique())
     mi_extra(file, HA_EXTRA_WRITE_CACHE, (void*) &size);
 
   can_enable_indexes= mi_is_all_keys_active(file->s->state.key_map,
@@ -1753,7 +1746,6 @@ void ha_myisam::start_bulk_insert(ha_rows rows, uint flags)
     if (file->open_flag & HA_OPEN_INTERNAL_TABLE)
     {
       file->update|= HA_STATE_CHANGED;
-      index_disabled= file->s->base.keys > 0;
       mi_clear_all_keys_active(file->s->state.key_map);
     }
     else
@@ -1784,7 +1776,6 @@ void ha_myisam::start_bulk_insert(ha_rows rows, uint flags)
             table->key_info[i].algorithm != HA_KEY_ALG_LONG_HASH)
         {
           mi_clear_key_active(share->state.key_map, i);
-          index_disabled= 1;
           file->update|= HA_STATE_CHANGED;
           file->create_unique_index_by_sort= all_keys;
         }
@@ -1792,15 +1783,12 @@ void ha_myisam::start_bulk_insert(ha_rows rows, uint flags)
     }
   }
   else
-  {
     if (!file->bulk_insert &&
         (!rows || rows >= MI_MIN_ROWS_TO_USE_BULK_INSERT))
     {
       mi_init_bulk_insert(file, (size_t) thd->variables.bulk_insert_buff_size,
                           rows);
     }
-  }
-  can_enable_indexes= index_disabled;
   DBUG_VOID_RETURN;
 }
 
@@ -1852,7 +1840,6 @@ int ha_myisam::end_bulk_insert()
         file->s->state.changed&= ~(STATE_CRASHED|STATE_CRASHED_ON_REPAIR);
       }
     }
-    can_enable_indexes= 0;
   }
   DBUG_PRINT("exit", ("first_error: %d", first_error));
   DBUG_RETURN(first_error);
@@ -2102,7 +2089,8 @@ int ha_myisam::info(uint flag)
 
     ref_length= misam_info.reflength;
     share->db_options_in_use= misam_info.options;
-    stats.block_size= myisam_block_size;        /* record block size */
+    /* record block size. We adjust with IO_SIZE to not make it too small */
+    stats.block_size= MY_MAX(myisam_block_size, IO_SIZE);
 
     if (table_share->tmp_table == NO_TMP_TABLE)
       mysql_mutex_lock(&table_share->LOCK_share);
@@ -2133,7 +2121,8 @@ int ha_myisam::info(uint flag)
 
 int ha_myisam::extra(enum ha_extra_function operation)
 {
-  if (operation == HA_EXTRA_MMAP && !opt_myisam_use_mmap)
+  if ((operation == HA_EXTRA_MMAP && !opt_myisam_use_mmap) ||
+      (operation == HA_EXTRA_WRITE_CACHE && has_long_unique()))
     return 0;
   return mi_extra(file, operation, 0);
 }
@@ -2349,6 +2338,8 @@ void ha_myisam::get_auto_increment(ulonglong offset, ulonglong increment,
     inx			Index to use
     min_key		Start of range.  Null pointer if from first key
     max_key		End of range. Null pointer if to last key
+    pages               Store first and last page for the range in case of
+                        b-trees. In other cases it's not touched.
 
   NOTES
     min_key.flag can have one of the following values:
@@ -2366,10 +2357,12 @@ void ha_myisam::get_auto_increment(ulonglong offset, ulonglong increment,
 			the range.
 */
 
-ha_rows ha_myisam::records_in_range(uint inx, key_range *min_key,
-                                    key_range *max_key)
+ha_rows ha_myisam::records_in_range(uint inx, const key_range *min_key,
+                                    const key_range *max_key,
+                                    page_range *pages)
 {
-  return (ha_rows) mi_records_in_range(file, (int) inx, min_key, max_key);
+  return (ha_rows) mi_records_in_range(file, (int) inx, min_key, max_key,
+                                       pages);
 }
 
 
@@ -2519,13 +2512,16 @@ int myisam_panic(handlerton *hton, ha_panic_function flag)
   return mi_panic(flag);
 }
 
+static int myisam_drop_table(handlerton *hton, const char *path)
+{
+  return mi_delete_table(path);
+}
+
 static int myisam_init(void *p)
 {
   handlerton *hton;
 
-#ifdef HAVE_PSI_INTERFACE
   init_myisam_psi_keys();
-#endif
 
   /* Set global variables based on startup options */
   if (myisam_recover_options && myisam_recover_options != HA_RECOVER_OFF)
@@ -2533,12 +2529,12 @@ static int myisam_init(void *p)
   else
     myisam_recover_options= HA_RECOVER_OFF;
 
-  myisam_block_size=(uint) 1 << my_bit_log2(opt_myisam_block_size);
+  myisam_block_size=(uint) 1 << my_bit_log2_uint64(opt_myisam_block_size);
 
   hton= (handlerton *)p;
-  hton->state= SHOW_OPTION_YES;
   hton->db_type= DB_TYPE_MYISAM;
   hton->create= myisam_create_handler;
+  hton->drop_table= myisam_drop_table;
   hton->panic= myisam_panic;
   hton->flags= HTON_CAN_RECREATE | HTON_SUPPORT_LOG_TABLES;
   hton->tablefile_extensions= ha_myisam_exts;
@@ -2652,23 +2648,6 @@ bool ha_myisam::rowid_filter_push(Rowid_filter* rowid_filter)
 struct st_mysql_storage_engine myisam_storage_engine=
 { MYSQL_HANDLERTON_INTERFACE_VERSION };
 
-mysql_declare_plugin(myisam)
-{
-  MYSQL_STORAGE_ENGINE_PLUGIN,
-  &myisam_storage_engine,
-  "MyISAM",
-  "MySQL AB",
-  "MyISAM storage engine",
-  PLUGIN_LICENSE_GPL,
-  myisam_init, /* Plugin Init */
-  NULL, /* Plugin Deinit */
-  0x0100, /* 1.0 */
-  NULL,                       /* status variables                */
-  myisam_sysvars,             /* system variables                */
-  NULL,
-  0,
-}
-mysql_declare_plugin_end;
 maria_declare_plugin(myisam)
 {
   MYSQL_STORAGE_ENGINE_PLUGIN,
