@@ -1,5 +1,5 @@
 /* Copyright (c) 2003, 2013, Oracle and/or its affiliates
-   Copyright (c) 2009, 2016, MariaDB
+   Copyright (c) 2009, 2020, MariaDB
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -35,7 +35,6 @@
 #define HAVE_CHARSET_mb2_or_mb4
 #endif
 
-
 #ifndef EILSEQ
 #define EILSEQ ENOENT
 #endif
@@ -143,12 +142,12 @@ my_copy_incomplete_char(CHARSET_INFO *cs,
     
     Make sure we didn't pad to an incorrect character.
   */
-  if (cs->cset->charlen(cs, (uchar *) dst, (uchar *) dst + cs->mbminlen) ==
+  if (my_ci_charlen(cs, (uchar *) dst, (uchar *) dst + cs->mbminlen) ==
       (int) cs->mbminlen)
     return MY_CHAR_COPY_OK;
 
   if (fix &&
-      cs->cset->wc_mb(cs, '?', (uchar *) dst, (uchar *) dst + cs->mbminlen) ==
+      my_ci_wc_mb(cs, '?', (uchar *) dst, (uchar *) dst + cs->mbminlen) ==
       (int) cs->mbminlen)
     return MY_CHAR_COPY_FIXED;
 
@@ -733,7 +732,7 @@ my_l10tostr_mb2_or_mb4(CHARSET_INFO *cs,
   
   for ( db= dst, de= dst + len ; (dst < de) && *p ; p++)
   {
-    int cnvres= cs->cset->wc_mb(cs,(my_wc_t)p[0],(uchar*) dst, (uchar*) de);
+    int cnvres= my_ci_wc_mb(cs, (my_wc_t) p[0], (uchar*) dst, (uchar*) de);
     if (cnvres > 0)
       dst+= cnvres;
     else
@@ -796,7 +795,7 @@ cnv:
   
   for ( db= dst, de= dst + len ; (dst < de) && *p ; p++)
   {
-    int cnvres= cs->cset->wc_mb(cs, (my_wc_t) p[0], (uchar*) dst, (uchar*) de);
+    int cnvres= my_ci_wc_mb(cs, (my_wc_t) p[0], (uchar*) dst, (uchar*) de);
     if (cnvres > 0)
       dst+= cnvres;
     else
@@ -829,7 +828,7 @@ my_strtoll10_mb2(CHARSET_INFO *cs __attribute__((unused)),
                  const char *nptr, char **endptr, int *error)
 {
   const uchar *s, *end, *start, *n_end, *true_end;
-  uchar UNINIT_VAR(c);
+  uchar c;
   unsigned long i, j, k;
   ulonglong li;
   int negative;
@@ -1057,7 +1056,7 @@ my_fill_mb2(CHARSET_INFO *cs, char *s, size_t slen, int fill)
 
   DBUG_ASSERT((slen % 2) == 0);
 
-  buflen= cs->cset->wc_mb(cs, (my_wc_t) fill, (uchar*) buf,
+  buflen= my_ci_wc_mb(cs, (my_wc_t) fill, (uchar*) buf,
                           (uchar*) buf + sizeof(buf));
 
   DBUG_ASSERT(buflen > 0);
@@ -1162,12 +1161,9 @@ static size_t
 my_snprintf_mb2(CHARSET_INFO *cs __attribute__((unused)),
                 char* to, size_t n, const char* fmt, ...)
 {
-  size_t ret;
   va_list args;
   va_start(args,fmt);
-  ret= my_vsnprintf_mb2(to, n, fmt, args);
-  va_end(args);
-  return ret;
+  return my_vsnprintf_mb2(to, n, fmt, args);
 }
 
 
@@ -1211,27 +1207,27 @@ static inline int my_weight_mb2_utf16mb2_general_ci(uchar b0, uchar b1)
 #define WEIGHT_ILSEQ(x)          (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB2(b0,b1)        my_weight_mb2_utf16mb2_general_ci(b0,b1)
 #define WEIGHT_MB4(b0,b1,b2,b3)  MY_CS_REPLACEMENT_CHARACTER
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 #define MY_FUNCTION_NAME(x)      my_ ## x ## _utf16_bin
 #define WEIGHT_ILSEQ(x)          (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB2(b0,b1)        ((int) MY_UTF16_WC2(b0, b1))
 #define WEIGHT_MB4(b0,b1,b2,b3)  ((int) MY_UTF16_WC4(b0, b1, b2, b3))
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 #define DEFINE_STRNNCOLLSP_NOPAD
 #define MY_FUNCTION_NAME(x)      my_ ## x ## _utf16_general_nopad_ci
 #define WEIGHT_ILSEQ(x)          (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB2(b0,b1)        my_weight_mb2_utf16mb2_general_ci(b0,b1)
 #define WEIGHT_MB4(b0,b1,b2,b3)  MY_CS_REPLACEMENT_CHARACTER
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 #define DEFINE_STRNNCOLLSP_NOPAD
 #define MY_FUNCTION_NAME(x)      my_ ## x ## _utf16_nopad_bin
 #define WEIGHT_ILSEQ(x)          (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB2(b0,b1)        ((int) MY_UTF16_WC2(b0, b1))
 #define WEIGHT_MB4(b0,b1,b2,b3)  ((int) MY_UTF16_WC4(b0, b1, b2, b3))
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 #undef IS_MB2_CHAR
 #undef IS_MB4_CHAR
@@ -1281,6 +1277,7 @@ my_uni_utf16(CHARSET_INFO *cs __attribute__((unused)),
 
 #ifdef HAVE_CHARSET_utf16
 
+const char charset_name_utf16le[]= "utf16le";
 
 static inline void
 my_tolower_utf16(MY_UNICASE_INFO *uni_plane, my_wc_t *wc)
@@ -1370,7 +1367,7 @@ static void
 my_hash_sort_utf16(CHARSET_INFO *cs, const uchar *s, size_t slen,
                    ulong *nr1, ulong *nr2)
 {
-  size_t lengthsp= cs->cset->lengthsp(cs, (const char *) s, slen);
+  size_t lengthsp= my_ci_lengthsp(cs, (const char *) s, slen);
   my_hash_sort_utf16_nopad(cs, s, lengthsp, nr1, nr2);
 }
 
@@ -1405,14 +1402,14 @@ static int
 my_charlen_utf16(CHARSET_INFO *cs, const uchar *str, const uchar *end)
 {
   my_wc_t wc;
-  return cs->cset->mb_wc(cs, &wc, str, end);
+  return my_ci_mb_wc(cs, &wc, str, end);
 }
 
 
 #define MY_FUNCTION_NAME(x)       my_ ## x ## _utf16
 #define CHARLEN(cs,str,end)       my_charlen_utf16(cs,str,end)
 #define DEFINE_WELL_FORMED_CHAR_LENGTH_USING_CHARLEN
-#include "ctype-mb.inl"
+#include "ctype-mb.ic"
 #undef MY_FUNCTION_NAME
 #undef CHARLEN
 #undef DEFINE_WELL_FORMED_CHAR_LENGTH_USING_CHARLEN
@@ -1495,7 +1492,7 @@ static void
 my_hash_sort_utf16_bin(CHARSET_INFO *cs,
                        const uchar *pos, size_t len, ulong *nr1, ulong *nr2)
 {
-  size_t lengthsp= cs->cset->lengthsp(cs, (const char *) pos, len);
+  size_t lengthsp= my_ci_lengthsp(cs, (const char *) pos, len);
   my_hash_sort_utf16_nopad_bin(cs, pos, lengthsp, nr1, nr2);
 }
 
@@ -1505,7 +1502,6 @@ static MY_COLLATION_HANDLER my_collation_utf16_general_ci_handler =
   NULL,                /* init */
   my_strnncoll_utf16_general_ci,
   my_strnncollsp_utf16_general_ci,
-  my_strnncollsp_nchars_utf16_general_ci,
   my_strnxfrm_utf16_general_ci,
   my_strnxfrmlen_unicode,
   my_like_range_generic,
@@ -1522,7 +1518,6 @@ static MY_COLLATION_HANDLER my_collation_utf16_bin_handler =
   NULL,                /* init */
   my_strnncoll_utf16_bin,
   my_strnncollsp_utf16_bin,
-  my_strnncollsp_nchars_utf16_bin,
   my_strnxfrm_unicode_full_bin,
   my_strnxfrmlen_unicode_full_bin,
   my_like_range_generic,
@@ -1539,7 +1534,6 @@ static MY_COLLATION_HANDLER my_collation_utf16_general_nopad_ci_handler =
   NULL,                /* init */
   my_strnncoll_utf16_general_ci,
   my_strnncollsp_utf16_general_nopad_ci,
-  my_strnncollsp_nchars_utf16_general_nopad_ci,
   my_strnxfrm_nopad_utf16_general_ci,
   my_strnxfrmlen_unicode,
   my_like_range_generic,
@@ -1556,7 +1550,6 @@ static MY_COLLATION_HANDLER my_collation_utf16_nopad_bin_handler =
   NULL,                /* init */
   my_strnncoll_utf16_bin,
   my_strnncollsp_utf16_nopad_bin,
-  my_strnncollsp_nchars_utf16_nopad_bin,
   my_strnxfrm_unicode_full_nopad_bin,
   my_strnxfrmlen_unicode_full_bin,
   my_like_range_generic,
@@ -1598,6 +1591,7 @@ MY_CHARSET_HANDLER my_charset_utf16_handler=
   my_well_formed_char_length_utf16,
   my_copy_fix_mb2_or_mb4,
   my_uni_utf16,
+  my_wc_to_printable_generic
 };
 
 
@@ -1605,7 +1599,7 @@ struct charset_info_st my_charset_utf16_general_ci=
 {
   54,0,0,              /* number       */
   MY_CS_COMPILED|MY_CS_PRIMARY|MY_CS_STRNXFRM|MY_CS_UNICODE|MY_CS_NONASCII,
-  "utf16",             /* cs name    */
+  charset_name_utf16,             /* cs name    */
   "utf16_general_ci",  /* name         */
   "UTF-16 Unicode",    /* comment      */
   NULL,                /* tailoring    */
@@ -1638,7 +1632,7 @@ struct charset_info_st my_charset_utf16_bin=
 {
   55,0,0,              /* number       */
   MY_CS_COMPILED|MY_CS_BINSORT|MY_CS_STRNXFRM|MY_CS_UNICODE|MY_CS_NONASCII,
-  "utf16",             /* cs name      */
+  charset_name_utf16,             /* cs name      */
   "utf16_bin",         /* name         */
   "UTF-16 Unicode",    /* comment      */
   NULL,                /* tailoring    */
@@ -1671,7 +1665,7 @@ struct charset_info_st my_charset_utf16_general_nopad_ci=
 {
   MY_NOPAD_ID(54),0,0, /* number           */
   MY_CS_COMPILED|MY_CS_STRNXFRM|MY_CS_UNICODE|MY_CS_NONASCII|MY_CS_NOPAD,
-  "utf16",             /* cs name          */
+  charset_name_utf16,             /* cs name          */
   "utf16_general_nopad_ci", /* name        */
   "UTF-16 Unicode",    /* comment          */
   NULL,                /* tailoring        */
@@ -1705,7 +1699,7 @@ struct charset_info_st my_charset_utf16_nopad_bin=
   MY_NOPAD_ID(55),0,0, /* number           */
   MY_CS_COMPILED|MY_CS_BINSORT|MY_CS_STRNXFRM|MY_CS_UNICODE|MY_CS_NONASCII|
   MY_CS_NOPAD,
-  "utf16",             /* cs name          */
+  charset_name_utf16,             /* cs name          */
   "utf16_nopad_bin",   /* name             */
   "UTF-16 Unicode",    /* comment          */
   NULL,                /* tailoring        */
@@ -1740,7 +1734,7 @@ struct charset_info_st my_charset_utf16_nopad_bin=
 #define MY_FUNCTION_NAME(x)      my_ ## x ## _utf16le_general_ci
 #define DEFINE_STRNXFRM_UNICODE
 #define DEFINE_STRNXFRM_UNICODE_NOPAD
-#define MY_MB_WC(cs, pwc, s, e)  (cs->cset->mb_wc(cs, pwc, s, e))
+#define MY_MB_WC(cs, pwc, s, e)  (my_ci_mb_wc(cs, pwc, s, e))
 #define OPTIMIZE_ASCII           0
 #define UNICASE_MAXCHAR          MY_UNICASE_INFO_DEFAULT_MAXCHAR
 #define UNICASE_PAGE0            my_unicase_default_page00
@@ -1748,27 +1742,27 @@ struct charset_info_st my_charset_utf16_nopad_bin=
 #define WEIGHT_ILSEQ(x)          (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB2(b0,b1)        my_weight_mb2_utf16mb2_general_ci(b1,b0)
 #define WEIGHT_MB4(b0,b1,b2,b3)  MY_CS_REPLACEMENT_CHARACTER
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 #define MY_FUNCTION_NAME(x)      my_ ## x ## _utf16le_bin
 #define WEIGHT_ILSEQ(x)          (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB2(b0,b1)        ((int) MY_UTF16_WC2(b1, b0))
 #define WEIGHT_MB4(b0,b1,b2,b3)  ((int) MY_UTF16_WC4(b1, b0, b3, b2))
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 #define DEFINE_STRNNCOLLSP_NOPAD
 #define MY_FUNCTION_NAME(x)      my_ ## x ## _utf16le_general_nopad_ci
 #define WEIGHT_ILSEQ(x)          (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB2(b0,b1)        my_weight_mb2_utf16mb2_general_ci(b1,b0)
 #define WEIGHT_MB4(b0,b1,b2,b3)  MY_CS_REPLACEMENT_CHARACTER
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 #define DEFINE_STRNNCOLLSP_NOPAD
 #define MY_FUNCTION_NAME(x)      my_ ## x ## _utf16le_nopad_bin
 #define WEIGHT_ILSEQ(x)          (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB2(b0,b1)        ((int) MY_UTF16_WC2(b1, b0))
 #define WEIGHT_MB4(b0,b1,b2,b3)  ((int) MY_UTF16_WC4(b1, b0, b3, b2))
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 #undef IS_MB2_CHAR
 #undef IS_MB4_CHAR
@@ -1849,7 +1843,6 @@ static MY_COLLATION_HANDLER my_collation_utf16le_general_ci_handler =
   NULL,                /* init */
   my_strnncoll_utf16le_general_ci,
   my_strnncollsp_utf16le_general_ci,
-  my_strnncollsp_nchars_utf16le_general_ci,
   my_strnxfrm_utf16le_general_ci,
   my_strnxfrmlen_unicode,
   my_like_range_generic,
@@ -1866,7 +1859,6 @@ static MY_COLLATION_HANDLER my_collation_utf16le_bin_handler =
   NULL,                /* init */
   my_strnncoll_utf16le_bin,
   my_strnncollsp_utf16le_bin,
-  my_strnncollsp_nchars_utf16le_bin,
   my_strnxfrm_unicode_full_bin,
   my_strnxfrmlen_unicode_full_bin,
   my_like_range_generic,
@@ -1883,7 +1875,6 @@ static MY_COLLATION_HANDLER my_collation_utf16le_general_nopad_ci_handler =
   NULL,                /* init */
   my_strnncoll_utf16le_general_ci,
   my_strnncollsp_utf16le_general_nopad_ci,
-  my_strnncollsp_nchars_utf16le_general_nopad_ci,
   my_strnxfrm_nopad_utf16le_general_ci,
   my_strnxfrmlen_unicode,
   my_like_range_generic,
@@ -1900,7 +1891,6 @@ static MY_COLLATION_HANDLER my_collation_utf16le_nopad_bin_handler =
   NULL,                /* init */
   my_strnncoll_utf16le_bin,
   my_strnncollsp_utf16le_nopad_bin,
-  my_strnncollsp_nchars_utf16le_nopad_bin,
   my_strnxfrm_unicode_full_nopad_bin,
   my_strnxfrmlen_unicode_full_bin,
   my_like_range_generic,
@@ -1942,6 +1932,7 @@ static MY_CHARSET_HANDLER my_charset_utf16le_handler=
   my_well_formed_char_length_utf16,
   my_copy_fix_mb2_or_mb4,
   my_uni_utf16le,
+  my_wc_to_printable_generic
 };
 
 
@@ -1949,7 +1940,7 @@ struct charset_info_st my_charset_utf16le_general_ci=
 {
   56,0,0,              /* number       */
   MY_CS_COMPILED|MY_CS_PRIMARY|MY_CS_STRNXFRM|MY_CS_UNICODE|MY_CS_NONASCII,
-  "utf16le",           /* cs name    */
+  charset_name_utf16le,           /* cs name    */
   "utf16le_general_ci",/* name         */
   "UTF-16LE Unicode",  /* comment      */
   NULL,                /* tailoring    */
@@ -1982,7 +1973,7 @@ struct charset_info_st my_charset_utf16le_bin=
 {
   62,0,0,              /* number       */
   MY_CS_COMPILED|MY_CS_BINSORT|MY_CS_STRNXFRM|MY_CS_UNICODE|MY_CS_NONASCII,
-  "utf16le",           /* cs name      */
+  charset_name_utf16le,           /* cs name      */
   "utf16le_bin",       /* name         */
   "UTF-16LE Unicode",  /* comment      */
   NULL,                /* tailoring    */
@@ -2015,7 +2006,7 @@ struct charset_info_st my_charset_utf16le_general_nopad_ci=
 {
   MY_NOPAD_ID(56),0,0, /* number           */
   MY_CS_COMPILED|MY_CS_STRNXFRM|MY_CS_UNICODE|MY_CS_NONASCII|MY_CS_NOPAD,
-  "utf16le",           /* cs name          */
+  charset_name_utf16le,           /* cs name          */
   "utf16le_general_nopad_ci",/* name       */
   "UTF-16LE Unicode",  /* comment          */
   NULL,                /* tailoring        */
@@ -2049,7 +2040,7 @@ struct charset_info_st my_charset_utf16le_nopad_bin=
   MY_NOPAD_ID(62),0,0, /* number           */
   MY_CS_COMPILED|MY_CS_BINSORT|MY_CS_STRNXFRM|MY_CS_UNICODE|MY_CS_NONASCII|
   MY_CS_NOPAD,
-  "utf16le",           /* cs name          */
+  charset_name_utf16le,           /* cs name          */
   "utf16le_nopad_bin", /* name             */
   "UTF-16LE Unicode",  /* comment          */
   NULL,                /* tailoring        */
@@ -2115,24 +2106,24 @@ static inline int my_weight_utf32_general_ci(uchar b0, uchar b1,
 #define UNICASE_PAGES            my_unicase_default_pages
 #define WEIGHT_ILSEQ(x)          (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB4(b0,b1,b2,b3)  my_weight_utf32_general_ci(b0, b1, b2, b3)
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 #define MY_FUNCTION_NAME(x)      my_ ## x ## _utf32_bin
 #define WEIGHT_ILSEQ(x)          (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB4(b0,b1,b2,b3)  ((int) MY_UTF32_WC4(b0, b1, b2, b3))
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 #define DEFINE_STRNNCOLLSP_NOPAD
 #define MY_FUNCTION_NAME(x)      my_ ## x ## _utf32_general_nopad_ci
 #define WEIGHT_ILSEQ(x)          (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB4(b0,b1,b2,b3)  my_weight_utf32_general_ci(b0, b1, b2, b3)
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 #define DEFINE_STRNNCOLLSP_NOPAD
 #define MY_FUNCTION_NAME(x)      my_ ## x ## _utf32_nopad_bin
 #define WEIGHT_ILSEQ(x)          (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB4(b0,b1,b2,b3)  ((int) MY_UTF32_WC4(b0, b1, b2, b3))
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 #undef IS_MB2_CHAR
 #undef IS_MB4_CHAR
@@ -2302,7 +2293,7 @@ my_charlen_utf32(CHARSET_INFO *cs __attribute__((unused)),
 #define MY_FUNCTION_NAME(x)       my_ ## x ## _utf32
 #define CHARLEN(cs,str,end)       my_charlen_utf32(cs,str,end)
 #define DEFINE_WELL_FORMED_CHAR_LENGTH_USING_CHARLEN
-#include "ctype-mb.inl"
+#include "ctype-mb.ic"
 #undef MY_FUNCTION_NAME
 #undef CHARLEN
 #undef DEFINE_WELL_FORMED_CHAR_LENGTH_USING_CHARLEN
@@ -2402,12 +2393,9 @@ static size_t
 my_snprintf_utf32(CHARSET_INFO *cs __attribute__((unused)),
                   char* to, size_t n, const char* fmt, ...)
 {
-  size_t ret;
   va_list args;
   va_start(args,fmt);
-  ret= my_vsnprintf_utf32(to, n, fmt, args);
-  va_end(args);
-  return ret;
+  return my_vsnprintf_utf32(to, n, fmt, args);
 }
 
 
@@ -2613,8 +2601,7 @@ void my_fill_utf32(CHARSET_INFO *cs,
 #ifdef DBUG_ASSERT_EXISTS
   buflen=
 #endif
-    cs->cset->wc_mb(cs, (my_wc_t) fill, (uchar*) buf,
-                    (uchar*) buf + sizeof(buf));
+    my_ci_wc_mb(cs, (my_wc_t) fill, (uchar*) buf, (uchar*) buf + sizeof(buf));
   DBUG_ASSERT(buflen == 4);
   while (s < e)
   {
@@ -2679,7 +2666,6 @@ static MY_COLLATION_HANDLER my_collation_utf32_general_ci_handler =
   NULL, /* init */
   my_strnncoll_utf32_general_ci,
   my_strnncollsp_utf32_general_ci,
-  my_strnncollsp_nchars_utf32_general_ci,
   my_strnxfrm_utf32_general_ci,
   my_strnxfrmlen_unicode,
   my_like_range_generic,
@@ -2696,7 +2682,6 @@ static MY_COLLATION_HANDLER my_collation_utf32_bin_handler =
   NULL, /* init */
   my_strnncoll_utf32_bin,
   my_strnncollsp_utf32_bin,
-  my_strnncollsp_nchars_utf32_bin,
   my_strnxfrm_unicode_full_bin,
   my_strnxfrmlen_unicode_full_bin,
   my_like_range_generic,
@@ -2713,7 +2698,6 @@ static MY_COLLATION_HANDLER my_collation_utf32_general_nopad_ci_handler =
   NULL, /* init */
   my_strnncoll_utf32_general_ci,
   my_strnncollsp_utf32_general_nopad_ci,
-  my_strnncollsp_nchars_utf32_general_nopad_ci,
   my_strnxfrm_nopad_utf32_general_ci,
   my_strnxfrmlen_unicode,
   my_like_range_generic,
@@ -2730,7 +2714,6 @@ static MY_COLLATION_HANDLER my_collation_utf32_nopad_bin_handler =
   NULL, /* init */
   my_strnncoll_utf32_bin,
   my_strnncollsp_utf32_nopad_bin,
-  my_strnncollsp_nchars_utf32_nopad_bin,
   my_strnxfrm_unicode_full_nopad_bin,
   my_strnxfrmlen_unicode_full_bin,
   my_like_range_generic,
@@ -2772,6 +2755,7 @@ MY_CHARSET_HANDLER my_charset_utf32_handler=
   my_well_formed_char_length_utf32,
   my_copy_fix_mb2_or_mb4,
   my_uni_utf32,
+  my_wc_to_printable_generic
 };
 
 
@@ -2779,7 +2763,7 @@ struct charset_info_st my_charset_utf32_general_ci=
 {
   60,0,0,              /* number       */
   MY_CS_COMPILED|MY_CS_PRIMARY|MY_CS_STRNXFRM|MY_CS_UNICODE|MY_CS_NONASCII,
-  "utf32",             /* cs name    */
+  charset_name_utf32,             /* cs name    */
   "utf32_general_ci",  /* name         */
   "UTF-32 Unicode",    /* comment      */
   NULL,                /* tailoring    */
@@ -2812,7 +2796,7 @@ struct charset_info_st my_charset_utf32_bin=
 {
   61,0,0,              /* number       */
   MY_CS_COMPILED|MY_CS_BINSORT|MY_CS_STRNXFRM|MY_CS_UNICODE|MY_CS_NONASCII,
-  "utf32",             /* cs name    */
+  charset_name_utf32,             /* cs name    */
   "utf32_bin",         /* name         */
   "UTF-32 Unicode",    /* comment      */
   NULL,                /* tailoring    */
@@ -2845,7 +2829,7 @@ struct charset_info_st my_charset_utf32_general_nopad_ci=
 {
   MY_NOPAD_ID(60),0,0, /* number           */
   MY_CS_COMPILED|MY_CS_STRNXFRM|MY_CS_UNICODE|MY_CS_NONASCII|MY_CS_NOPAD,
-  "utf32",             /* cs name          */
+  charset_name_utf32,             /* cs name          */
   "utf32_general_nopad_ci", /* name        */
   "UTF-32 Unicode",    /* comment          */
   NULL,                /* tailoring        */
@@ -2879,7 +2863,7 @@ struct charset_info_st my_charset_utf32_nopad_bin=
   MY_NOPAD_ID(61),0,0, /* number           */
   MY_CS_COMPILED|MY_CS_BINSORT|MY_CS_STRNXFRM|MY_CS_UNICODE|MY_CS_NONASCII|
   MY_CS_NOPAD,
-  "utf32",             /* cs name          */
+  charset_name_utf32,             /* cs name          */
   "utf32_nopad_bin",   /* name             */
   "UTF-32 Unicode",    /* comment          */
   NULL,                /* tailoring        */
@@ -2974,7 +2958,7 @@ static const uchar to_upper_ucs2[] = {
 };
 
 
-/* Definitions for strcoll.inl */
+/* Definitions for strcoll.ic */
 #define IS_MB2_CHAR(x,y)            (1)
 #define UCS2_CODE(b0,b1)            (((uchar) b0) << 8 | ((uchar) b1))
 
@@ -2997,7 +2981,7 @@ static inline int my_weight_mb2_ucs2_general_ci(uchar b0, uchar b1)
 #define UNICASE_PAGES            my_unicase_default_pages
 #define WEIGHT_ILSEQ(x)          (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB2(b0,b1)        my_weight_mb2_ucs2_general_ci(b0,b1)
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 
 #define MY_FUNCTION_NAME(x)      my_ ## x ## _ucs2_bin
@@ -3006,21 +2990,21 @@ static inline int my_weight_mb2_ucs2_general_ci(uchar b0, uchar b1)
 #define OPTIMIZE_ASCII           0
 #define WEIGHT_ILSEQ(x)          (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB2(b0,b1)        UCS2_CODE(b0,b1)
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 
 #define DEFINE_STRNNCOLLSP_NOPAD
 #define MY_FUNCTION_NAME(x)    my_ ## x ## _ucs2_general_nopad_ci
 #define WEIGHT_ILSEQ(x)        (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB2(b0,b1)      my_weight_mb2_ucs2_general_ci(b0,b1)
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 
 #define DEFINE_STRNNCOLLSP_NOPAD
 #define MY_FUNCTION_NAME(x)    my_ ## x ## _ucs2_nopad_bin
 #define WEIGHT_ILSEQ(x)        (0xFF0000 + (uchar) (x))
 #define WEIGHT_MB2(b0,b1)      UCS2_CODE(b0,b1)
-#include "strcoll.inl"
+#include "strcoll.ic"
 
 
 static int
@@ -3273,7 +3257,6 @@ static MY_COLLATION_HANDLER my_collation_ucs2_general_ci_handler =
     NULL,		/* init */
     my_strnncoll_ucs2_general_ci,
     my_strnncollsp_ucs2_general_ci,
-    my_strnncollsp_nchars_ucs2_general_ci,
     my_strnxfrm_ucs2_general_ci,
     my_strnxfrmlen_unicode,
     my_like_range_generic,
@@ -3290,7 +3273,6 @@ static MY_COLLATION_HANDLER my_collation_ucs2_bin_handler =
     NULL,		/* init */
     my_strnncoll_ucs2_bin,
     my_strnncollsp_ucs2_bin,
-    my_strnncollsp_nchars_ucs2_bin,
     my_strnxfrm_ucs2_bin,
     my_strnxfrmlen_unicode,
     my_like_range_generic,
@@ -3307,7 +3289,6 @@ static MY_COLLATION_HANDLER my_collation_ucs2_general_nopad_ci_handler =
     NULL,		/* init */
     my_strnncoll_ucs2_general_ci,
     my_strnncollsp_ucs2_general_nopad_ci,
-    my_strnncollsp_nchars_ucs2_general_nopad_ci,
     my_strnxfrm_nopad_ucs2_general_ci,
     my_strnxfrmlen_unicode,
     my_like_range_generic,
@@ -3324,7 +3305,6 @@ static MY_COLLATION_HANDLER my_collation_ucs2_nopad_bin_handler =
     NULL,		/* init */
     my_strnncoll_ucs2_bin,
     my_strnncollsp_ucs2_nopad_bin,
-    my_strnncollsp_nchars_ucs2_nopad_bin,
     my_strnxfrm_nopad_ucs2_bin,
     my_strnxfrmlen_unicode,
     my_like_range_generic,
@@ -3366,6 +3346,7 @@ MY_CHARSET_HANDLER my_charset_ucs2_handler=
     my_well_formed_char_length_ucs2,
     my_copy_fix_mb2_or_mb4,
     my_uni_ucs2,
+    my_wc_to_printable_generic
 };
 
 
@@ -3373,7 +3354,7 @@ struct charset_info_st my_charset_ucs2_general_ci=
 {
     35,0,0,		/* number       */
     MY_CS_COMPILED|MY_CS_PRIMARY|MY_CS_STRNXFRM|MY_CS_UNICODE|MY_CS_NONASCII,
-    "ucs2",		/* cs name    */
+    charset_name_ucs2,		/* cs name    */
     "ucs2_general_ci",	/* name         */
     "",			/* comment      */
     NULL,		/* tailoring    */
@@ -3406,7 +3387,7 @@ struct charset_info_st my_charset_ucs2_general_mysql500_ci=
 {
   159, 0, 0,                                       /* number           */
   MY_CS_COMPILED|MY_CS_STRNXFRM|MY_CS_UNICODE|MY_CS_NONASCII, /* state */
-  "ucs2",                                          /* cs name          */
+  charset_name_ucs2,                                          /* cs name          */
   "ucs2_general_mysql500_ci",                      /* name             */
   "",                                              /* comment          */
   NULL,                                            /* tailoring        */
@@ -3439,7 +3420,7 @@ struct charset_info_st my_charset_ucs2_bin=
 {
     90,0,0,		/* number       */
     MY_CS_COMPILED|MY_CS_BINSORT|MY_CS_UNICODE|MY_CS_NONASCII,
-    "ucs2",		/* cs name    */
+    charset_name_ucs2,		/* cs name    */
     "ucs2_bin",		/* name         */
     "",			/* comment      */
     NULL,		/* tailoring    */
@@ -3472,7 +3453,7 @@ struct charset_info_st my_charset_ucs2_general_nopad_ci=
 {
     MY_NOPAD_ID(35),0,0,     /* number           */
     MY_CS_COMPILED|MY_CS_STRNXFRM|MY_CS_UNICODE|MY_CS_NONASCII|MY_CS_NOPAD,
-    "ucs2",                  /* cs name          */
+    charset_name_ucs2,                  /* cs name          */
     "ucs2_general_nopad_ci", /* name             */
     "",                      /* comment          */
     NULL,                    /* tailoring        */
@@ -3505,7 +3486,7 @@ struct charset_info_st my_charset_ucs2_nopad_bin=
 {
     MY_NOPAD_ID(90),0,0,     /* number           */
     MY_CS_COMPILED|MY_CS_BINSORT|MY_CS_UNICODE|MY_CS_NONASCII|MY_CS_NOPAD,
-    "ucs2",                  /* cs name          */
+    charset_name_ucs2,                  /* cs name          */
     "ucs2_nopad_bin",        /* name             */
     "",                      /* comment          */
     NULL,                    /* tailoring        */
