@@ -28,6 +28,7 @@
 #ifdef __WIN__
   #include <winsock2.h>
   #include <MSWSock.h>
+  #include <mstcpip.h>
   #pragma comment(lib, "ws2_32.lib")
 #endif
 #include "my_context.h"
@@ -73,8 +74,7 @@ int vio_errno(Vio *vio __attribute__((unused)))
 {
   /* These transport types are not Winsock based. */
 #ifdef _WIN32
-  if (vio->type == VIO_TYPE_NAMEDPIPE ||
-      vio->type == VIO_TYPE_SHARED_MEMORY)
+  if (vio->type == VIO_TYPE_NAMEDPIPE)
     return GetLastError();
 #endif
 
@@ -82,6 +82,16 @@ int vio_errno(Vio *vio __attribute__((unused)))
   return socket_errno;
 }
 
+static int vio_set_linger(my_socket s, unsigned short timeout_sec)
+{
+  struct linger s_linger;
+  int ret;
+  s_linger.l_onoff = 1;
+  s_linger.l_linger = timeout_sec;
+  ret = setsockopt(s, SOL_SOCKET, SO_LINGER, (const char *)&s_linger, (int)sizeof(s_linger));
+  return ret;
+}
+
 
 /**
   Attempt to wait for an I/O event on a socket.
@@ -114,6 +124,7 @@ int vio_socket_io_wait(Vio *vio, enum enum_vio_io_event event)
   case  0:
     /* The wait timed out. */
     ret= -1;
+    vio_set_linger(vio->mysql_socket.fd, 0);
     break;
   default:
     /* A positive value indicates an I/O event. */
@@ -137,6 +148,10 @@ int vio_socket_io_wait(Vio *vio, enum enum_vio_io_event event)
 #define VIO_DONTWAIT 0
 #endif
 
+#ifndef SOCKET_EAGAIN
+#define SOCKET_EAGAIN SOCKET_EWOULDBLOCK
+#endif
+
 /*
   returns number of bytes read or -1 in case of an error
 */
@@ -351,7 +366,7 @@ int vio_blocking(Vio *vio, my_bool set_blocking_mode, my_bool *old_mode)
   r= set_blocking_mode ? 0 : 1;
 #endif /* !defined(NO_FCNTL_NONBLOCK) */
 #else /* !defined(__WIN__) */
-  if (vio->type != VIO_TYPE_NAMEDPIPE && vio->type != VIO_TYPE_SHARED_MEMORY)
+  if (vio->type != VIO_TYPE_NAMEDPIPE)
   { 
     ulong arg;
     int old_fcntl=vio->fcntl_mode;
@@ -399,43 +414,6 @@ int vio_socket_timeout(Vio *vio,
 {
   int ret= 0;
   DBUG_ENTER("vio_socket_timeout");
-
-#if defined(_WIN32)
-  {
-    int optname;
-    DWORD timeout= 0;
-    const char *optval= (const char *) &timeout;
-
-    /*
-      The default socket timeout value is zero, which means an infinite
-      timeout. Values less than 500 milliseconds are interpreted to be of
-      500 milliseconds. Hence, the VIO behavior for zero timeout, which is
-      intended to cause the send or receive operation to fail immediately
-      if no data is available, is not supported on WIN32 and neither is
-      necessary as it's not possible to set the VIO timeout value to zero.
-
-      Assert that the VIO timeout is either positive or set to infinite.
-    */
-    DBUG_ASSERT(which || vio->read_timeout);
-    DBUG_ASSERT(!which || vio->write_timeout);
-
-    if (which)
-    {
-      optname= SO_SNDTIMEO;
-      if (vio->write_timeout > 0)
-        timeout= vio->write_timeout;
-    }
-    else
-    {
-      optname= SO_RCVTIMEO;
-      if (vio->read_timeout > 0)
-        timeout= vio->read_timeout;
-    }
-
-    ret= mysql_socket_setsockopt(vio->mysql_socket, SOL_SOCKET, optname,
-	                             optval, sizeof(timeout));
-  }
-#else
   /*
     The MSG_DONTWAIT trick is not used with SSL sockets as the send and
     receive I/O operations are wrapped through SSL-specific functions
@@ -456,18 +434,43 @@ int vio_socket_timeout(Vio *vio,
     if (new_mode != old_mode)
       ret= vio_blocking(vio, new_mode, &not_used);
   }
-#endif
 
   DBUG_RETURN(ret);
 }
 
+/* Set TCP_NODELAY (disable Nagle's algorithm */
+int vio_nodelay(Vio *vio, my_bool on)
+{
+  int r;
+  int no_delay= MY_TEST(on);
+  DBUG_ENTER("vio_nodelay");
 
-int vio_fastsend(Vio * vio __attribute__((unused)))
+  if (vio->type == VIO_TYPE_NAMEDPIPE || vio->type == VIO_TYPE_SOCKET)
+  {
+    DBUG_RETURN(0);
+  }
+
+  r = mysql_socket_setsockopt(vio->mysql_socket, IPPROTO_TCP, TCP_NODELAY,
+    IF_WIN((const char*), (void*)) &no_delay,
+    sizeof(no_delay));
+
+  if (r)
+  {
+    DBUG_PRINT("warning",
+     ("Couldn't set socket option for fast send, error %d",
+      socket_errno));
+     r = -1;
+  }
+  DBUG_PRINT("exit", ("%d", r));
+  DBUG_RETURN(r);
+}
+
+int vio_fastsend(Vio * vio)
 {
   int r=0;
   DBUG_ENTER("vio_fastsend");
 
-  if (vio->type == VIO_TYPE_NAMEDPIPE ||vio->type == VIO_TYPE_SHARED_MEMORY)
+  if (vio->type == VIO_TYPE_NAMEDPIPE)
   {
     DBUG_RETURN(0);
   }
@@ -480,18 +483,7 @@ int vio_fastsend(Vio * vio __attribute__((unused)))
   }
 #endif                                    /* IPTOS_THROUGHPUT */
   if (!r)
-  {
-#ifdef __WIN__
-    BOOL nodelay= 1;
-#else
-    int nodelay = 1;
-#endif
-
-    r= mysql_socket_setsockopt(vio->mysql_socket, IPPROTO_TCP, TCP_NODELAY,
-                  IF_WIN((const char*), (void*)) &nodelay,
-                  sizeof(nodelay));
-
-  }
+    r = vio_nodelay(vio, TRUE);
   if (r)
   {
     DBUG_PRINT("warning",
@@ -512,7 +504,7 @@ int vio_keepalive(Vio* vio, my_bool set_keep_alive)
                        (int)mysql_socket_getfd(vio->mysql_socket),
                        (int)set_keep_alive));
 
-  if (vio->type != VIO_TYPE_NAMEDPIPE && vio->type != VIO_TYPE_SHARED_MEMORY)
+  if (vio->type != VIO_TYPE_NAMEDPIPE)
   {
     if (set_keep_alive)
       opt = 1;
@@ -522,6 +514,63 @@ int vio_keepalive(Vio* vio, my_bool set_keep_alive)
   DBUG_RETURN(r);
 }
 
+/*
+  Set socket options for keepalive e.g., TCP_KEEPCNT, TCP_KEEPIDLE/TCP_KEEPALIVE, TCP_KEEPINTVL
+*/
+int vio_set_keepalive_options(Vio* vio, const struct vio_keepalive_opts *opts)
+{
+#if defined _WIN32
+  struct tcp_keepalive s;
+  DWORD  nbytes;
+
+  if (vio->type == VIO_TYPE_NAMEDPIPE)
+    return 0;
+
+  if (!opts->idle && !opts->interval)
+    return 0;
+
+  s.onoff= 1;
+  s.keepalivetime= opts->idle? opts->idle * 1000 : 7200;
+  s.keepaliveinterval= opts->interval?opts->interval * 1000 : 1;
+
+  return WSAIoctl(vio->mysql_socket.fd, SIO_KEEPALIVE_VALS, (LPVOID) &s, sizeof(s),
+           NULL, 0, &nbytes, NULL, NULL);
+
+#elif defined (TCP_KEEPIDLE) || defined (TCP_KEEPALIVE)
+
+  int ret= 0;
+  if (opts->idle)
+  {
+#ifdef TCP_KEEPIDLE // Linux only
+    ret= mysql_socket_setsockopt(vio->mysql_socket, IPPROTO_TCP, TCP_KEEPIDLE, (char *)&opts->idle, sizeof(opts->idle));
+#elif defined (TCP_KEEPALIVE)
+    ret= mysql_socket_setsockopt(vio->mysql_socket, IPPROTO_TCP, TCP_KEEPALIVE, (char *)&opts->idle, sizeof(opts->idle));
+#endif
+    if(ret)
+      return ret;
+  }
+
+#ifdef TCP_KEEPCNT // Linux only
+  if(opts->probes)
+  {
+    ret= mysql_socket_setsockopt(vio->mysql_socket, IPPROTO_TCP, TCP_KEEPCNT, (char *)&opts->probes, sizeof(opts->probes));
+    if(ret)
+      return ret;
+  }
+#endif
+
+#ifdef TCP_KEEPINTVL  // Linux only
+  if(opts->interval)
+  {
+    ret= mysql_socket_setsockopt(vio->mysql_socket, IPPROTO_TCP, TCP_KEEPINTVL, (char *)&opts->interval, sizeof(opts->interval));
+  }
+#endif
+  return ret;
+#else /*TCP_KEEPIDLE || TCP_KEEPALIVE */
+  return -1; 
+#endif
+}
+
 
 /**
   Indicate whether a I/O operation must be retried later.
@@ -572,8 +621,6 @@ int vio_close(Vio *vio)
       vio->type == VIO_TYPE_SSL);
 
     DBUG_ASSERT(mysql_socket_getfd(vio->mysql_socket) >= 0);
-    if (mysql_socket_shutdown(vio->mysql_socket, SHUT_RDWR))
-      r= -1;
     if (mysql_socket_close(vio->mysql_socket))
       r= -1;
   }
@@ -598,6 +645,26 @@ enum enum_vio_type vio_type(Vio* vio)
   return vio->type;
 }
 
+static const LEX_CSTRING vio_type_names[] =
+{
+  { STRING_WITH_LEN("Error") }, // cannot happen
+  { STRING_WITH_LEN("TCP/IP") },
+  { STRING_WITH_LEN("Socket") },
+  { STRING_WITH_LEN("Named Pipe") },
+  { STRING_WITH_LEN("SSL/TLS") },
+  { STRING_WITH_LEN("Shared Memory") }
+};
+
+const char *vio_type_name(enum enum_vio_type vio_type, size_t *len)
+{
+  int index= vio_type >= FIRST_VIO_TYPE && vio_type <= LAST_VIO_TYPE
+             ?  vio_type : 0;
+
+  *len= vio_type_names[index].length;
+  return vio_type_names[index].str;
+}
+
+
 my_socket vio_fd(Vio* vio)
 {
   return mysql_socket_getfd(vio->mysql_socket);
@@ -624,18 +691,15 @@ my_socket vio_fd(Vio* vio)
   @param src_length [in] length of the src.
   @param dst        [out] a buffer to store normalized IP address
                           (sockaddr_storage).
-  @param dst_length [out] actual length of the normalized IP address.
+  @param dst_length [out] optional - actual length of the normalized IP address.
 */
 
-static void vio_get_normalized_ip(const struct sockaddr *src,
-                                  int src_length,
-                                  struct sockaddr *dst,
-                                  int *dst_length)
+void vio_get_normalized_ip(const struct sockaddr *src, size_t src_length,
+                                  struct sockaddr *dst)
 {
   switch (src->sa_family) {
   case AF_INET:
     memcpy(dst, src, src_length);
-    *dst_length= src_length;
     break;
 
 #ifdef HAVE_IPV6
@@ -654,9 +718,7 @@ static void vio_get_normalized_ip(const struct sockaddr *src,
         be converted to the IPv4 form.
       */
 
-      *dst_length= sizeof (struct sockaddr_in);
-
-      memset(dst_ip4, 0, *dst_length);
+      memset(dst_ip4, 0, sizeof (struct sockaddr_in));
       dst_ip4->sin_family= AF_INET;
       dst_ip4->sin_port= src_addr6->sin6_port;
 
@@ -670,9 +732,7 @@ static void vio_get_normalized_ip(const struct sockaddr *src,
     else
     {
       /* This is a "native" IPv6 address. */
-
       memcpy(dst, src, src_length);
-      *dst_length= src_length;
     }
 
     break;
@@ -703,17 +763,15 @@ static void vio_get_normalized_ip(const struct sockaddr *src,
   @retval FALSE on success.
 */
 
-my_bool vio_get_normalized_ip_string(const struct sockaddr *addr,
-                                     int addr_length,
+my_bool vio_get_normalized_ip_string(const struct sockaddr *addr, size_t addr_length,
                                      char *ip_string,
                                      size_t ip_string_size)
 {
   struct sockaddr_storage norm_addr_storage;
   struct sockaddr *norm_addr= (struct sockaddr *) &norm_addr_storage;
-  int norm_addr_length;
   int err_code;
 
-  vio_get_normalized_ip(addr, addr_length, norm_addr, &norm_addr_length);
+  vio_get_normalized_ip(addr, addr_length, norm_addr);
 
   err_code= vio_getnameinfo(norm_addr, ip_string, ip_string_size, NULL, 0,
                             NI_NUMERICHOST);
@@ -752,9 +810,7 @@ my_bool vio_peer_addr(Vio *vio, char *ip_buffer, uint16 *port,
       address.
     */
     struct in_addr *ip4= &((struct sockaddr_in *) &(vio->remote))->sin_addr;
-
     vio->remote.ss_family= AF_INET;
-    vio->addrLen= sizeof (struct sockaddr_in);
 
     ip4->s_addr= htonl(INADDR_LOOPBACK);
 
@@ -771,7 +827,6 @@ my_bool vio_peer_addr(Vio *vio, char *ip_buffer, uint16 *port,
     struct sockaddr_storage addr_storage;
     struct sockaddr *addr= (struct sockaddr *) &addr_storage;
     size_socket addr_length= sizeof (addr_storage);
-
     /* Get sockaddr by socked fd. */
 
     err_code= mysql_socket_getpeername(vio->mysql_socket, addr, &addr_length);
@@ -785,7 +840,7 @@ my_bool vio_peer_addr(Vio *vio, char *ip_buffer, uint16 *port,
     /* Normalize IP address. */
 
     vio_get_normalized_ip(addr, addr_length,
-                          (struct sockaddr *) &vio->remote, &vio->addrLen);
+                          (struct sockaddr *) &vio->remote);
 
     /* Get IP address & port number. */
 
@@ -826,7 +881,7 @@ static my_bool socket_peek_read(Vio *vio, uint *bytes)
 {
   my_socket sd= mysql_socket_getfd(vio->mysql_socket);
 #if defined(_WIN32)
-  int len;
+  u_long len;
   if (ioctlsocket(sd, FIONREAD, &len))
     return TRUE;
   *bytes= len;
@@ -1197,7 +1252,6 @@ my_bool vio_is_connected(Vio *vio)
   DBUG_RETURN(bytes ? TRUE : FALSE);
 }
 
-#ifndef DBUG_OFF
 
 /**
   Number of bytes in the read or socket buffer
@@ -1216,22 +1270,34 @@ ssize_t vio_pending(Vio *vio)
     return vio->read_end - vio->read_pos;
 
   /* Skip non-socket based transport types. */
-  if (vio->type == VIO_TYPE_TCPIP || vio->type == VIO_TYPE_SOCKET)
+  switch (vio->type)
   {
+  case VIO_TYPE_TCPIP:
+    /* fallthrough */
+  case VIO_TYPE_SOCKET:
     /* Obtain number of readable bytes in the socket buffer. */
     if (socket_peek_read(vio, &bytes))
       return -1;
+    return bytes;
+
+  case VIO_TYPE_SSL:
+    bytes= (uint) SSL_pending(vio->ssl_arg);
+    if (bytes)
+      return bytes;
+    if (socket_peek_read(vio, &bytes))
+      return -1;
+    return bytes;
+
+#ifdef _WIN32
+  case VIO_TYPE_NAMEDPIPE:
+    bytes= vio_pending_pipe(vio);
+    return bytes;
+#endif
+  default:
+    return -1;
   }
-
-  /*
-    SSL not checked due to a yaSSL bug in SSL_pending that
-    causes it to attempt to read from the socket.
-  */
-
-  return (ssize_t) bytes;
 }
 
-#endif /* DBUG_OFF */
 
 /**
   Checks if the error code, returned by vio_getnameinfo(), means it was the
